@@ -1,36 +1,57 @@
-"""This module handles direct I/O operations for working with .slp files."""
+"""This module handles direct I/O operations for working with .slp files.
+
+Format version history:
+    - 1.0: Initial format
+    - 1.1: Changed coordinate system from top-left pixel at (0, 0) to center at (0, 0)
+    - 1.2: Added tracking_score field to instances
+    - 1.3: Added explicit handling for tracking_score
+    - 1.4: Added channel_order attribute to embedded video datasets to track RGB vs BGR
+"""
 
 from __future__ import annotations
-import numpy as np
+
+import sys
+from enum import Enum, IntEnum
+from pathlib import Path
+from typing import TYPE_CHECKING, Optional, Union
+
 import h5py
+import imageio.v3 as iio
+import numpy as np
 import simplejson as json
-from typing import Union, Optional
-from sleap_io import (
-    Video,
-    Skeleton,
-    Edge,
-    Symmetry,
-    Node,
-    Track,
-    SuggestionFrame,
-    Instance,
-    PredictedInstance,
-    LabeledFrame,
-    Labels,
+from tqdm import tqdm
+
+from sleap_io.io.skeleton import SkeletonSLPDecoder, SkeletonSLPEncoder
+from sleap_io.io.utils import (
+    is_file_accessible,
+    read_hdf5_attrs,
+    read_hdf5_dataset,
+    sanitize_filename,
+)
+from sleap_io.io.video_reading import (
+    HDF5Video,
+    ImageVideo,
+    MediaVideo,
+    TiffVideo,
+    VideoBackend,
+)
+from sleap_io.model.camera import (
     Camera,
     CameraGroup,
-    InstanceGroup,
     FrameGroup,
+    InstanceGroup,
     RecordingSession,
 )
-from sleap_io.io.video_reading import VideoBackend, ImageVideo, MediaVideo, HDF5Video
-from sleap_io.io.utils import read_hdf5_attrs, read_hdf5_dataset, is_file_accessible
-from sleap_io.io.skeleton import SkeletonSLPDecoder, SkeletonSLPEncoder
-from enum import IntEnum, Enum
-from pathlib import Path
-import imageio.v3 as iio
-import sys
-from tqdm import tqdm
+from sleap_io.model.instance import Instance, PredictedInstance, Track
+from sleap_io.model.labeled_frame import LabeledFrame
+from sleap_io.model.labels import Labels
+from sleap_io.model.skeleton import Skeleton
+from sleap_io.model.suggestions import SuggestionFrame
+from sleap_io.model.video import Video
+
+if TYPE_CHECKING:
+    from sleap_io.model.instance import PointsArray, PredictedPointsArray
+    from sleap_io.model.labels_set import LabelsSet
 
 try:
     import cv2
@@ -53,23 +74,6 @@ class InstanceType(IntEnum):
     PREDICTED = 1
 
 
-def sanitize_filename(
-    filename: str | Path | list[str] | list[Path],
-) -> str | list[str]:
-    """Sanitize a filename to a canonical posix-compatible format.
-
-    Args:
-        filename: A string or `Path` object or list of either to sanitize.
-
-    Returns:
-        A sanitized filename as a string (or list of strings if a list was provided)
-        with forward slashes and posix-formatted.
-    """
-    if isinstance(filename, list):
-        return [sanitize_filename(f) for f in filename]
-    return Path(filename).as_posix().replace("\\", "/")
-
-
 def make_video(
     labels_path: str,
     video_json: dict,
@@ -177,6 +181,7 @@ def make_video(
                 dataset=backend_metadata.get("dataset", None),
                 grayscale=grayscale,
                 input_format=backend_metadata.get("input_format", None),
+                format=backend_metadata.get("format", None),
             )
         except Exception:
             backend = None
@@ -229,7 +234,7 @@ def video_to_dict(video: Video, labels_path: Optional[str] = None) -> dict:
     # Add backend metadata
     if video.backend is None:
         result["backend"] = video.backend_metadata
-    elif type(video.backend) == MediaVideo:
+    elif type(video.backend) is MediaVideo:
         result["backend"] = {
             "type": "MediaVideo",
             "shape": video.shape,
@@ -239,7 +244,7 @@ def video_to_dict(video: Video, labels_path: Optional[str] = None) -> dict:
             "dataset": "",
             "input_format": "",
         }
-    elif type(video.backend) == HDF5Video:
+    elif type(video.backend) is HDF5Video:
         # Determine if we should use self-reference or external reference
         use_self_reference = (
             video.backend.has_embedded_images
@@ -258,7 +263,7 @@ def video_to_dict(video: Video, labels_path: Optional[str] = None) -> dict:
             "has_embedded_images": video.backend.has_embedded_images,
             "grayscale": video.grayscale,
         }
-    elif type(video.backend) == ImageVideo:
+    elif type(video.backend) is ImageVideo:
         if video.shape is not None:
             height, width, channels = video.shape[1:4]
         else:
@@ -273,6 +278,15 @@ def video_to_dict(video: Video, labels_path: Optional[str] = None) -> dict:
             "channels_": channels,
             "grayscale": video.grayscale,
         }
+    elif type(video.backend) is TiffVideo:
+        result["backend"] = {
+            "type": "TiffVideo",
+            "shape": video.shape,
+            "filename": video_filename,
+            "grayscale": video.grayscale,
+            "keep_open": video.backend.keep_open,
+            "format": video.backend.format,
+        }
 
     # Add source_video metadata if present
     if hasattr(video, "source_video") and video.source_video is not None:
@@ -285,128 +299,6 @@ def video_to_dict(video: Video, labels_path: Optional[str] = None) -> dict:
     return result
 
 
-def embed_video(
-    labels_path: str,
-    video: Video,
-    group: str,
-    frame_inds: list[int],
-    image_format: str = "png",
-    fixed_length: bool = True,
-) -> Video:
-    """Embed frames of a video in a SLEAP labels file.
-
-    .. deprecated:: 1.0.0
-        This function is deprecated. Use `process_and_embed_frames` instead.
-
-    Args:
-        labels_path: A string path to the SLEAP labels file.
-        video: A `Video` object to embed in the labels file.
-        group: The name of the group to store the embedded video in. Image data will be
-            stored in a dataset named `{group}/video`. Frame indices will be stored
-            in a data set named `{group}/frame_numbers`.
-        frame_inds: A list of frame indices to embed.
-        image_format: The image format to use for embedding. Valid formats are "png"
-            (the default), "jpg" or "hdf5".
-        fixed_length: If `True` (the default), the embedded images will be padded to the
-            length of the largest image. If `False`, the images will be stored as
-            variable length, which is smaller but may not be supported by all readers.
-
-    Returns:
-        An embedded `Video` object.
-
-        If the video is already embedded, the original video will be returned. If not,
-        a new `Video` object will be created with the embedded data.
-    """
-    # Load the image data and optionally encode it.
-    imgs_data = []
-    for frame_idx in frame_inds:
-        frame = video[frame_idx]
-
-        if image_format == "hdf5":
-            img_data = frame
-        else:
-            if "cv2" in sys.modules:
-                img_data = np.squeeze(
-                    cv2.imencode("." + image_format, frame)[1]
-                ).astype("int8")
-            else:
-                if frame.shape[-1] == 1:
-                    frame = frame.squeeze(axis=-1)
-                img_data = np.frombuffer(
-                    iio.imwrite("<bytes>", frame, extension="." + image_format),
-                    dtype="int8",
-                )
-
-        imgs_data.append(img_data)
-
-    # Write the image data to the labels file.
-    with h5py.File(labels_path, "a") as f:
-        if image_format == "hdf5":
-            f.create_dataset(
-                f"{group}/video", data=imgs_data, compression="gzip", chunks=True
-            )
-        else:
-            if fixed_length:
-                img_bytes_len = 0
-                for img in imgs_data:
-                    img_bytes_len = max(img_bytes_len, len(img))
-                ds = f.create_dataset(
-                    f"{group}/video",
-                    shape=(len(imgs_data), img_bytes_len),
-                    dtype="int8",
-                    compression="gzip",
-                )
-                for i, img in enumerate(imgs_data):
-                    ds[i, : len(img)] = img
-            else:
-                ds = f.create_dataset(
-                    f"{group}/video",
-                    shape=(len(imgs_data),),
-                    dtype=h5py.special_dtype(vlen=np.dtype("int8")),
-                )
-                for i, img in enumerate(imgs_data):
-                    ds[i] = img
-
-        # Store metadata.
-        ds.attrs["format"] = image_format
-        video_shape = video.shape
-        (
-            ds.attrs["frames"],
-            ds.attrs["height"],
-            ds.attrs["width"],
-            ds.attrs["channels"],
-        ) = video_shape
-
-        # Store frame indices.
-        f.create_dataset(f"{group}/frame_numbers", data=frame_inds)
-
-        # Store source video.
-        if video.source_video is not None:
-            # If this is already an embedded dataset, retain the previous source video.
-            source_video = video.source_video
-        else:
-            source_video = video
-
-        # Create a new video object with the embedded data.
-        embedded_video = Video(
-            filename=labels_path,
-            backend=VideoBackend.from_filename(
-                labels_path,
-                dataset=f"{group}/video",
-                grayscale=video.grayscale,
-                keep_open=False,
-            ),
-            source_video=source_video,
-        )
-
-        grp = f.require_group(f"{group}/source_video")
-        grp.attrs["json"] = json.dumps(
-            video_to_dict(source_video, labels_path), separators=(",", ":")
-        )
-
-    return embedded_video
-
-
 def prepare_frames_to_embed(
     labels_path: str,
     labels: Labels,
@@ -417,7 +309,8 @@ def prepare_frames_to_embed(
     Args:
         labels_path: A string path to the SLEAP labels file.
         labels: A `Labels` object containing the videos.
-        frames_to_embed: A list of tuples of `(video, frame_idx)` specifying the frames to embed.
+        frames_to_embed: A list of tuples of `(video, frame_idx)` specifying the
+            frames to embed.
 
     Returns:
         A list of dictionaries, each containing metadata for a frame to embed:
@@ -461,6 +354,7 @@ def process_and_embed_frames(
     image_format: str = "png",
     fixed_length: bool = True,
     verbose: bool = True,
+    plugin: Optional[str] = None,
 ) -> dict[Video, Video]:
     """Process and embed frames into a SLEAP labels file.
 
@@ -469,17 +363,31 @@ def process_and_embed_frames(
 
     Args:
         labels_path: A string path to the SLEAP labels file.
-        frames_metadata: A list of dictionaries with frame metadata from prepare_frames_to_embed.
+        frames_metadata: A list of dictionaries with frame metadata from
+            prepare_frames_to_embed.
         image_format: The image format to use for embedding. Valid formats are "png"
             (the default), "jpg" or "hdf5".
         fixed_length: If `True` (the default), the embedded images will be padded to the
             length of the largest image. If `False`, the images will be stored as
             variable length, which is smaller but may not be supported by all readers.
-        verbose: If `True` (the default), display a progress bar for the embedding process.
+        verbose: If `True` (the default), display a progress bar for the embedding
+            process.
+        plugin: Image plugin to use for encoding. One of "opencv" or "imageio".
+            If None, uses the global default from `get_default_image_plugin()`.
+            If no global default is set, auto-detects based on available packages.
 
     Returns:
         A dictionary mapping original Video objects to their embedded versions.
     """
+    # Determine which plugin to use for encoding
+    from sleap_io.io.video_reading import get_default_image_plugin
+
+    if plugin is None:
+        plugin = get_default_image_plugin()
+    if plugin is None:
+        # Auto-detect: prefer opencv, fallback to imageio
+        plugin = "opencv" if "cv2" in sys.modules else "imageio"
+
     # Initialize a dictionary to store data by group
     data_by_group = {}
 
@@ -500,6 +408,7 @@ def process_and_embed_frames(
                 "video": video,  # All frames in a group are from the same video
                 "frame_inds": [],
                 "imgs_data": [],
+                "channel_order": None,  # Track channel order: "RGB" or "BGR"
             }
 
         # Load the frame
@@ -508,18 +417,25 @@ def process_and_embed_frames(
         # Encode the frame
         if image_format == "hdf5":
             img_data = frame
+            channel_order = "RGB"  # HDF5 format stores as-is (RGB)
         else:
-            if "cv2" in sys.modules:
+            if plugin == "opencv":
                 img_data = np.squeeze(
                     cv2.imencode("." + image_format, frame)[1]
                 ).astype("int8")
-            else:
+                channel_order = "BGR"  # OpenCV encodes in BGR
+            else:  # imageio
                 if frame.shape[-1] == 1:
                     frame = frame.squeeze(axis=-1)
                 img_data = np.frombuffer(
                     iio.imwrite("<bytes>", frame, extension="." + image_format),
                     dtype="int8",
                 )
+                channel_order = "RGB"  # imageio encodes in RGB
+
+        # Store channel order (should be consistent for all frames in a group)
+        if data_by_group[group]["channel_order"] is None:
+            data_by_group[group]["channel_order"] = channel_order
 
         # Store frame data in the appropriate group
         data_by_group[group]["imgs_data"].append(img_data)
@@ -562,6 +478,7 @@ def process_and_embed_frames(
 
             # Store metadata
             ds.attrs["format"] = image_format
+            ds.attrs["channel_order"] = data["channel_order"]
             video_shape = video.shape
             (
                 ds.attrs["frames"],
@@ -609,6 +526,7 @@ def embed_frames(
     embed: list[tuple[Video, int]],
     image_format: str = "png",
     verbose: bool = True,
+    plugin: Optional[str] = None,
 ):
     """Embed frames in a SLEAP labels file.
 
@@ -618,7 +536,10 @@ def embed_frames(
         embed: A list of tuples of `(video, frame_idx)` specifying the frames to embed.
         image_format: The image format to use for embedding. Valid formats are "png"
             (the default), "jpg" or "hdf5".
-        verbose: If `True` (the default), display a progress bar for the embedding process.
+        verbose: If `True` (the default), display a progress bar for the embedding
+            process.
+        plugin: Image plugin to use for encoding. One of "opencv" or "imageio".
+            If None, uses the global default from `get_default_image_plugin()`.
 
     Notes:
         This function will embed the frames in the labels file and update the `Videos`
@@ -626,7 +547,11 @@ def embed_frames(
     """
     frames_metadata = prepare_frames_to_embed(labels_path, labels, embed)
     replaced_videos = process_and_embed_frames(
-        labels_path, frames_metadata, image_format=image_format, verbose=verbose
+        labels_path,
+        frames_metadata,
+        image_format=image_format,
+        verbose=verbose,
+        plugin=plugin,
     )
 
     if len(replaced_videos) > 0:
@@ -638,6 +563,7 @@ def embed_videos(
     labels: Labels,
     embed: bool | str | list[tuple[Video, int]],
     verbose: bool = True,
+    plugin: Optional[str] = None,
 ):
     """Embed videos in a SLEAP labels file.
 
@@ -653,7 +579,10 @@ def embed_videos(
 
             If `True` or `"all"`, all labeled frames and suggested frames will be
             embedded.
-        verbose: If `True` (the default), display a progress bar for the embedding process.
+        verbose: If `True` (the default), display a progress bar for the embedding
+            process.
+        plugin: Image plugin to use for encoding. One of "opencv" or "imageio".
+            If None, uses the global default from `get_default_image_plugin()`.
 
             If `"source"` is specified, no images will be embedded and the source video
             will be restored if available.
@@ -679,7 +608,7 @@ def embed_videos(
     else:
         raise ValueError(f"Invalid value for embed: {embed}")
 
-    embed_frames(labels_path, labels, embed, verbose=verbose)
+    embed_frames(labels_path, labels, embed, verbose=verbose, plugin=plugin)
 
 
 def write_videos(
@@ -697,7 +626,8 @@ def write_videos(
         videos: A list of `Video` objects to store the metadata for.
         restore_source: Deprecated. Use reference_mode instead. If `True`, restore
             source videos if available and will not re-embed the embedded images.
-            If `False` (the default), will re-embed images that were previously embedded.
+            If `False` (the default), will re-embed images that were previously
+            embedded.
         reference_mode: How to handle video references:
             - EMBED: Re-embed frames that were previously embedded
             - RESTORE_ORIGINAL: Use original video if available
@@ -718,10 +648,11 @@ def write_videos(
 
     # First determine which videos need embedding
     for video_ind, video in enumerate(videos):
-        if type(video.backend) == HDF5Video and video.backend.has_embedded_images:
+        if type(video.backend) is HDF5Video and video.backend.has_embedded_images:
             if reference_mode == VideoReferenceMode.RESTORE_ORIGINAL:
                 if video.source_video is None:
-                    # No source video available, reference the current embedded video file
+                    # No source video available, reference the current embedded video
+                    # file
                     videos_to_write.append((video_ind, video))
                 else:
                     # Use the source video
@@ -770,9 +701,7 @@ def write_videos(
             image_format=[
                 v.backend.image_format if hasattr(v.backend, "image_format") else "png"
                 for _, v, _ in videos_to_embed
-            ][
-                0
-            ],  # Use the first video's format
+            ][0],  # Use the first video's format
             verbose=verbose,
         )
 
@@ -799,7 +728,8 @@ def write_videos(
             # If original_videos is provided (e.g., during embedding), use those
             original_video = original_videos[video_ind] if original_videos else video
 
-            # Determine what metadata to save based on reference mode and video structure
+            # Determine what metadata to save based on reference mode and video
+            # structure
             original_to_save = None
             source_to_save = None
 
@@ -812,19 +742,22 @@ def write_videos(
                     and hasattr(original_video.source_video, "original_video")
                     and original_video.source_video.original_video is not None
                 ):
-                    # If source_video has original_video, use that (it's the true original)
+                    # If source_video has original_video, use that (it's the true
+                    # original)
                     original_to_save = original_video.source_video.original_video
                 elif (
                     original_video.source_video is not None
                     and reference_mode == VideoReferenceMode.EMBED
                 ):
-                    # For embed mode, if we only have source_video, that becomes the original
+                    # For embed mode, if we only have source_video, that becomes the
+                    # original
                     original_to_save = original_video.source_video
 
             # Handle source_video metadata
             if reference_mode != VideoReferenceMode.PRESERVE_SOURCE:
                 if reference_mode == VideoReferenceMode.EMBED and original_videos:
-                    # For embed mode, save the original video as source (it's the .pkg.slp)
+                    # For embed mode, save the original video as source (it's the
+                    # .pkg.slp)
                     source_to_save = original_video
                 elif original_video.source_video is not None:
                     source_to_save = original_video.source_video
@@ -834,7 +767,8 @@ def write_videos(
                 video_group = f[dataset]
 
                 if original_to_save is not None:
-                    # Store original_video metadata as a group (consistent with source_video)
+                    # Store original_video metadata as a group (consistent with
+                    # source_video)
                     original_grp = video_group.require_group("original_video")
                     original_json = video_to_dict(original_to_save, labels_path)
                     original_grp.attrs["json"] = json.dumps(
@@ -842,7 +776,8 @@ def write_videos(
                     )
 
                 if source_to_save is not None:
-                    # For EMBED mode with original_videos, we need to overwrite source_video
+                    # For EMBED mode with original_videos, we need to overwrite
+                    # source_video
                     # because embed_videos saves the wrong metadata
                     if (
                         reference_mode == VideoReferenceMode.EMBED
@@ -911,10 +846,14 @@ def read_suggestions(labels_path: str, videos: list[Video]) -> list[SuggestionFr
     suggestions = [json.loads(x) for x in suggestions]
     suggestions_objects = []
     for suggestion in suggestions:
+        # Extract metadata (e.g., "group")
+        metadata = {"group": suggestion.get("group", 0)}
+
         suggestions_objects.append(
             SuggestionFrame(
                 video=videos[int(suggestion["video"])],
                 frame_idx=suggestion["frame_idx"],
+                metadata=metadata,
             )
         )
     return suggestions_objects
@@ -930,13 +869,15 @@ def write_suggestions(
         suggestions: A list of `SuggestionFrame` objects to store the metadata for.
         videos: A list of `Video` objects.
     """
-    GROUP = 0  # TODO: Handle storing extraneous metadata.
     suggestions_json = []
     for suggestion in suggestions:
+        # Get group from metadata if available, otherwise use default
+        group = suggestion.metadata.get("group", 0) if suggestion.metadata else 0
+
         suggestion_dict = {
             "video": str(videos.index(suggestion.video)),
             "frame_idx": suggestion.frame_idx,
-            "group": GROUP,
+            "group": group,
         }
         suggestion_json = np.bytes_(json.dumps(suggestion_dict, separators=(",", ":")))
         suggestions_json.append(suggestion_json)
@@ -1038,7 +979,7 @@ def write_metadata(labels_path: str, labels: Labels):
 
     with h5py.File(labels_path, "a") as f:
         grp = f.require_group("metadata")
-        grp.attrs["format_id"] = 1.3
+        grp.attrs["format_id"] = 1.4
         grp.attrs["json"] = np.bytes_(json.dumps(md, separators=(",", ":")))
 
 
@@ -1068,6 +1009,45 @@ def read_pred_points(labels_path: str) -> np.ndarray:
     return pred_pts
 
 
+def _points_from_hdf5_data(
+    pts_data: np.ndarray,
+    skeleton: Skeleton,
+    is_predicted: bool = False,
+) -> Union["PointsArray", "PredictedPointsArray"]:
+    """Build PointsArray directly from HDF5 structured array data.
+
+    This is a fast path that avoids column_stack and intermediate array creation
+    by directly constructing the target PointsArray structure from HDF5 data.
+
+    Args:
+        pts_data: Structured array from HDF5 with fields x, y, visible, complete,
+            and optionally score.
+        skeleton: The skeleton defining the node structure.
+        is_predicted: If True, create a PredictedPointsArray with scores.
+
+    Returns:
+        A fully populated PointsArray or PredictedPointsArray.
+    """
+    from sleap_io.model.instance import PointsArray, PredictedPointsArray
+
+    n = len(pts_data)
+
+    if is_predicted:
+        points = PredictedPointsArray.empty(n)
+        points["score"] = pts_data["score"]
+    else:
+        points = PointsArray.empty(n)
+
+    # Direct field assignment (faster than column_stack)
+    points["xy"][:, 0] = pts_data["x"]
+    points["xy"][:, 1] = pts_data["y"]
+    points["visible"] = pts_data["visible"]
+    points["complete"] = pts_data["complete"]
+    points["name"] = skeleton.node_names
+
+    return points
+
+
 def read_instances(
     labels_path: str,
     skeletons: list[Skeleton],
@@ -1128,28 +1108,37 @@ def read_instances(
 
         if instance_type == InstanceType.USER:
             pts_data = points[point_id_start:point_id_end]
+            # Fast path: Build PointsArray directly from HDF5 data
+            points_array = _points_from_hdf5_data(
+                pts_data, skeleton, is_predicted=False
+            )
+            if format_id < 1.1:
+                # Legacy coordinate system: top-left of pixel is (0, 0)
+                # Adjust to new system: center of pixel is (0, 0)
+                points_array["xy"] -= 0.5
             inst = Instance(
-                np.column_stack([pts_data["x"], pts_data["y"]]),
+                points_array,
                 skeleton=skeleton,
                 track=track,
                 tracking_score=tracking_score,
             )
-            inst.points["visible"] = pts_data["visible"]
-            inst.points["complete"] = pts_data["complete"]
             instances[instance_id] = inst
 
         elif instance_type == InstanceType.PREDICTED:
             pts_data = pred_points[point_id_start:point_id_end]
+            # Fast path: Build PredictedPointsArray directly from HDF5 data
+            points_array = _points_from_hdf5_data(pts_data, skeleton, is_predicted=True)
+            if format_id < 1.1:
+                # Legacy coordinate system: top-left of pixel is (0, 0)
+                # Adjust to new system: center of pixel is (0, 0)
+                points_array["xy"] -= 0.5
             inst = PredictedInstance(
-                np.column_stack([pts_data["x"], pts_data["y"]]),
+                points_array,
                 skeleton=skeleton,
                 track=track,
                 score=instance_score,
                 tracking_score=tracking_score,
             )
-            inst.points["score"] = pts_data["score"]
-            inst.points["visible"] = pts_data["visible"]
-            inst.points["complete"] = pts_data["complete"]
             instances[instance_id] = inst
 
         if from_predicted >= 0:
@@ -1220,7 +1209,7 @@ def write_lfs(labels_path: str, labels: Labels):
                 to_link.append((instance_id, inst.from_predicted))
             score = 0.0
 
-            if type(inst) == Instance:
+            if type(inst) is Instance:
                 instance_type = InstanceType.USER
                 tracking_score = inst.tracking_score
                 point_id_start = len(points)
@@ -1232,7 +1221,7 @@ def write_lfs(labels_path: str, labels: Labels):
 
                 point_id_end = len(points)
 
-            elif type(inst) == PredictedInstance:
+            elif type(inst) is PredictedInstance:
                 instance_type = InstanceType.PREDICTED
                 score = inst.score
                 tracking_score = inst.tracking_score
@@ -1394,7 +1383,7 @@ def make_frame_group(
             and optional keys:
             - "frame_idx": Frame index.
             - Any keys containing metadata.
-        labeled_frames_list: List of `LabeledFrame` objects (expecting
+        labeled_frames: List of `LabeledFrame` objects (expecting
             `Labels.labeled_frames`).
         camera_group: `CameraGroup` object used to retrieve `Camera` objects.
 
@@ -1529,8 +1518,8 @@ def make_session(
             - "frame_group_dicts": List of dictionaries containing `FrameGroup`
                 information. See `make_frame_group` for what each dictionary contains.
             - Any optional keys containing metadata.
-        videos_list: List containing `Video` objects (expected `Labels.videos`).
-        labeled_frames_list: List containing `LabeledFrame` objects (expected
+        videos: List containing `Video` objects (expected `Labels.videos`).
+        labeled_frames: List containing `LabeledFrame` objects (expected
             `Labels.labeled_frames`).
 
     Returns:
@@ -1569,8 +1558,7 @@ def make_session(
             frame_group_by_frame_idx[frame_group.frame_idx] = frame_group
         except ValueError as e:
             print(
-                f"Error reconstructing FrameGroup: {frame_group_dict}. Skipping..."
-                f"\n{e}"
+                f"Error reconstructing FrameGroup: {frame_group_dict}. Skipping...\n{e}"
             )
 
     session = RecordingSession(
@@ -1708,7 +1696,8 @@ def camera_to_dict(camera: Camera) -> dict:
     Returns:
         Dictionary containing camera information with the following keys:
             - "name": Camera name.
-            - "size": Image size (width, height) of camera in pixels of size (2,) and type
+            - "size": Image size (width, height) of camera in pixels of size (2,) and
+              type
                 int.
             - "matrix": Intrinsic camera matrix of size (3, 3) and type float64.
             - "distortions": Radial-tangential distortion coefficients
@@ -1925,12 +1914,75 @@ def read_labels(labels_path: str, open_videos: bool = True) -> Labels:
     return labels
 
 
+def read_labels_set(
+    path: Union[str, Path, list[Union[str, Path]], dict[str, Union[str, Path]]],
+    open_videos: bool = True,
+) -> LabelsSet:
+    """Load a LabelsSet from multiple SLP files.
+
+    Args:
+        path: Can be one of:
+            - A directory path containing .slp files
+            - A list of .slp file paths
+            - A dictionary mapping names to .slp file paths
+        open_videos: If `True` (the default), attempt to open the video backend for
+            I/O. If `False`, the backend will not be opened.
+
+    Returns:
+        A LabelsSet containing the loaded Labels objects.
+
+    Examples:
+        Load from directory:
+        >>> labels_set = read_labels_set("path/to/splits/")
+
+        Load from list:
+        >>> labels_set = read_labels_set(["train.slp", "val.slp", "test.slp"])
+
+        Load from dictionary:
+        >>> labels_set = read_labels_set({"train": "train.slp", "val": "val.slp"})
+    """
+    from sleap_io.model.labels_set import LabelsSet
+
+    labels_dict = {}
+
+    if isinstance(path, dict):
+        # Dictionary of name -> path mappings
+        for name, file_path in path.items():
+            labels_dict[name] = read_labels(str(file_path), open_videos=open_videos)
+
+    elif isinstance(path, list):
+        # List of paths - auto-generate names
+        for i, file_path in enumerate(path):
+            file_path = Path(file_path)
+            # Use filename without extension as key, or fall back to generic name
+            name = file_path.stem if file_path.stem else f"labels_{i}"
+            labels_dict[name] = read_labels(str(file_path), open_videos=open_videos)
+
+    else:
+        # Directory path - find all .slp files
+        path = Path(path)
+        if not path.is_dir():
+            raise ValueError(f"Path must be a directory, list, or dict. Got: {path}")
+
+        slp_files = sorted(path.glob("*.slp"))
+        if not slp_files:
+            raise ValueError(f"No .slp files found in directory: {path}")
+
+        for slp_file in slp_files:
+            # Use filename without extension as key
+            name = slp_file.stem
+            labels_dict[name] = read_labels(str(slp_file), open_videos=open_videos)
+
+    return LabelsSet(labels=labels_dict)
+
+
 def write_labels(
     labels_path: str,
     labels: Labels,
     embed: bool | str | list[tuple[Video, int]] | None = None,
     restore_original_videos: bool = True,
     verbose: bool = True,
+    plugin: Optional[str] = None,
 ):
     """Write a SLEAP labels file.
 
@@ -1955,16 +2007,20 @@ def write_labels(
             video files. If `False` and `embed=False`, keep references to source
             `.pkg.slp` files. Only applies when `embed=False`.
         verbose: If `True` (the default), display a progress bar when embedding frames.
+        plugin: Image plugin to use for encoding embedded frames. One of "opencv"
+            or "imageio". If None, uses the global default from
+            `get_default_image_plugin()`. If no global default is set, auto-detects
+            based on available packages.
     """
     if Path(labels_path).exists():
         Path(labels_path).unlink()
 
     # Store original videos before embedding modifies them
     # We need to make a copy of the actual video objects, not just the list
-    original_videos = [v for v in labels.videos] if embed and embed != False else None
+    original_videos = [v for v in labels.videos] if embed else None
 
-    if embed and embed != False:
-        embed_videos(labels_path, labels, embed, verbose=verbose)
+    if embed:
+        embed_videos(labels_path, labels, embed, verbose=verbose, plugin=plugin)
 
     # Determine reference mode based on parameters
     if embed == "source" or (embed is False and restore_original_videos):
