@@ -1,4 +1,4 @@
-"""This module handles direct I/O operations for working with Ultralytics YOLO pose format.
+"""Handles direct I/O operations for working with Ultralytics YOLO pose format.
 
 Ultralytics YOLO pose format specification:
 - Directory structure: dataset_root/split/images/ and dataset_root/split/labels/
@@ -10,21 +10,26 @@ Ultralytics YOLO pose format specification:
 """
 
 from __future__ import annotations
+
+import multiprocessing
+import warnings
+from multiprocessing import Pool
+from pathlib import Path
+from typing import TYPE_CHECKING, Dict, List, Optional, Tuple
+
+import imageio.v3 as iio
 import numpy as np
 import yaml
-from typing import Dict, List, Optional, Union, Tuple
-from pathlib import Path
-import warnings
-from sleap_io.model.video import Video
-from sleap_io.model.skeleton import Skeleton, Edge, Node
-from sleap_io.model.instance import Track, Instance
+from tqdm import tqdm
+
+from sleap_io.model.instance import Instance, Track
 from sleap_io.model.labeled_frame import LabeledFrame
 from sleap_io.model.labels import Labels
-import imageio.v3 as iio
-from tqdm import tqdm
-import multiprocessing
-from multiprocessing import Pool
-from functools import partial
+from sleap_io.model.skeleton import Edge, Node, Skeleton
+from sleap_io.model.video import Video
+
+if TYPE_CHECKING:
+    from sleap_io.model.labels_set import LabelsSet
 
 
 def read_labels(
@@ -36,11 +41,14 @@ def read_labels(
     """Read Ultralytics YOLO pose dataset and return a `Labels` object.
 
     Args:
-        dataset_path: Path to the Ultralytics dataset root directory containing data.yaml.
+        dataset_path: Path to the Ultralytics dataset root directory containing
+            data.yaml.
         split: Dataset split to read ('train', 'val', or 'test'). Defaults to 'train'.
-        skeleton: Optional skeleton to use. If not provided, will be inferred from data.yaml.
+        skeleton: Optional skeleton to use. If not provided, will be inferred from
+            data.yaml.
         image_size: Image dimensions (height, width) for coordinate denormalization.
-                   Defaults to (480, 640). Will attempt to infer from actual images if available.
+                   Defaults to (480, 640). Will attempt to infer from actual images if
+                   available.
 
     Returns:
         Parsed labels as a `Labels` instance.
@@ -88,7 +96,8 @@ def read_labels(
             # Parse label file if it exists
             instances = []
             if label_file.exists():
-                # Get image dimensions - try from video shape first, fallback to reading image
+                # Get image dimensions - try from video shape first, fallback to reading
+                # image
                 if video.shape is not None:
                     img_shape = video.shape[:2]
                 else:
@@ -117,6 +126,103 @@ def read_labels(
     )
 
 
+def read_labels_set(
+    dataset_path: str,
+    splits: Optional[List[str]] = None,
+    skeleton: Optional[Skeleton] = None,
+    image_size: Tuple[int, int] = (480, 640),
+) -> LabelsSet:
+    """Read multiple splits from an Ultralytics dataset as a LabelsSet.
+
+    Args:
+        dataset_path: Path to the root directory of the Ultralytics dataset.
+        splits: List of split names to load (e.g., ["train", "val", "test"]).
+            If None, will attempt to load all available splits.
+        skeleton: Skeleton to use for the dataset. If None, will attempt to
+            load from data.yaml file in the dataset root.
+        image_size: Default image size (height, width) to use if unable to
+            determine from the actual images. Default: (480, 640).
+
+    Returns:
+        A LabelsSet containing Labels objects for each split.
+
+    Example:
+        >>> labels_set = read_labels_set("path/to/yolo_dataset/")
+        >>> train_labels = labels_set["train"]
+        >>> val_labels = labels_set["val"]
+    """
+    from sleap_io.model.labels_set import LabelsSet
+
+    dataset_path = Path(dataset_path)
+
+    # If no splits specified, try to detect available splits
+    if splits is None:
+        splits = []
+        for split_name in ["train", "val", "test", "valid"]:
+            if (dataset_path / split_name).exists():
+                splits.append(split_name)
+
+        if not splits:
+            raise ValueError(f"No splits found in dataset path: {dataset_path}")
+
+    # Try to load skeleton from data.yaml if not provided
+    if skeleton is None:
+        data_yaml_path = dataset_path / "data.yaml"
+        if data_yaml_path.exists():
+            with open(data_yaml_path, "r") as f:
+                data_config = yaml.safe_load(f)
+
+            # Try to create skeleton from our custom metadata first
+            if "node_names" in data_config and "skeleton" in data_config:
+                try:
+                    node_names = data_config["node_names"]
+                    skeleton_connections = data_config["skeleton"]
+
+                    # Create nodes from names
+                    nodes = [Node(name) for name in node_names]
+
+                    # Create edges from skeleton connections
+                    edges = []
+                    for connection in skeleton_connections:
+                        if len(connection) == 2:
+                            src_idx, dst_idx = connection
+                            if 0 <= src_idx < len(nodes) and 0 <= dst_idx < len(nodes):
+                                edges.append(Edge(nodes[src_idx], nodes[dst_idx]))
+
+                    skeleton = Skeleton(nodes=nodes, edges=edges)
+                except (KeyError, IndexError, TypeError):
+                    # Fall back to basic skeleton creation
+                    pass
+
+            # Fall back to basic skeleton creation if metadata approach failed
+            if skeleton is None and "kpt_shape" in data_config:
+                kpt_shape = data_config["kpt_shape"]
+                if isinstance(kpt_shape, list) and len(kpt_shape) >= 2:
+                    n_keypoints = kpt_shape[0]
+                    # Create a basic skeleton with numbered nodes
+                    nodes = [Node(name=str(i)) for i in range(n_keypoints)]
+                    skeleton = Skeleton(nodes=nodes)
+
+    labels_dict = {}
+
+    for split in splits:
+        try:
+            labels = read_labels(
+                dataset_path=str(dataset_path),
+                split=split,
+                skeleton=skeleton,
+                image_size=image_size,
+            )
+            labels_dict[split] = labels
+        except Exception:
+            continue
+
+    if not labels_dict:
+        raise ValueError(f"Could not load any splits from dataset: {dataset_path}")
+
+    return LabelsSet(labels=labels_dict)
+
+
 def _save_frame_image(args: Tuple[dict, str, Optional[int]]) -> Optional[str]:
     """Worker function to save a single frame image.
 
@@ -183,12 +289,16 @@ def write_labels(
         dataset_path: Path to write the Ultralytics dataset.
         split_ratios: Dictionary mapping split names to ratios (must sum to 1.0).
         class_id: Class ID to use for all instances (default: 0).
-        image_format: Image format to use for saving frames. Either "png" (default, lossless) or "jpg".
-        image_quality: Image quality for JPEG format (1-100). For PNG, this is the compression level (0-9).
+        image_format: Image format to use for saving frames. Either "png" (default,
+            lossless) or "jpg".
+        image_quality: Image quality for JPEG format (1-100). For PNG, this is the
+            compression level (0-9).
             If None, uses default quality settings.
         verbose: If True (default), show progress bars during export.
-        use_multiprocessing: If True, use multiprocessing for parallel image saving. Default is False.
-        n_workers: Number of worker processes. If None, uses CPU count - 1. Only used if use_multiprocessing=True.
+        use_multiprocessing: If True, use multiprocessing for parallel image saving.
+            Default is False.
+        n_workers: Number of worker processes. If None, uses CPU count - 1. Only used
+            if use_multiprocessing=True.
         **kwargs: Additional arguments (unused, for compatibility).
     """
     dataset_path = Path(dataset_path)
@@ -289,7 +399,8 @@ def write_labels(
                     frame_img = frame.image
                     if frame_img is None:
                         warnings.warn(
-                            f"Could not load frame {frame.frame_idx} from video, skipping."
+                            f"Could not load frame {frame.frame_idx} from video, "
+                            f"skipping."
                         )
                         continue
 
@@ -379,7 +490,7 @@ def parse_label_file(
                     )
                     continue
 
-                class_id = int(parts[0])
+                _ = int(parts[0])  # class_id - not used but part of YOLO format
                 x_center, y_center, width, height = map(float, parts[1:5])
 
                 # Parse keypoints
@@ -560,7 +671,7 @@ def create_splits_from_labels(
                 n_train=train_ratio, n_val=val_ratio, n_test=test_ratio
             )
             return {"train": train_split, "val": val_split, "test": test_split}
-        except:
+        except Exception:
             # Fallback to manual splitting
             first_split = train_ratio + val_ratio
             temp_split, test_split = labels.split(first_split)
