@@ -8,12 +8,14 @@ estimated, such as confidence scores.
 """
 
 from __future__ import annotations
+
+from typing import Optional, Union
+
 import attrs
-from typing import ClassVar, Optional, Union
-from sleap_io import Skeleton, Node
-from sleap_io.model.skeleton import NodeOrIndex
 import numpy as np
 
+from sleap_io.model.skeleton import Node, Skeleton
+
 
 class PointsArray(np.ndarray):
     """A specialized array for storing instance points data.
@@ -36,17 +38,21 @@ class PointsArray(np.ndarray):
             np.dtype: A structured numpy dtype with fields for xy coordinates,
                 visible flag, complete flag, and node names.
         """
-        return np.dtype(
-            [
-                ("xy", "<f8", (2,)),  # 64-bit (8-byte) little-endian double, ndim=2
-                ("visible", "bool"),
-                ("complete", "bool"),
-                (
-                    "name",
-                    "O",
-                ),  # object dtype to store pointers to python string objects
-            ]
-        )
+        # Cache the dtype at the class level for performance
+        # Use cls.__dict__ to check if defined on this class (not inherited)
+        if "_cached_dtype" not in cls.__dict__:
+            cls._cached_dtype = np.dtype(
+                [
+                    ("xy", "<f8", (2,)),  # 64-bit (8-byte) little-endian double, ndim=2
+                    ("visible", "bool"),
+                    ("complete", "bool"),
+                    (
+                        "name",
+                        "O",
+                    ),  # object dtype to store pointers to python string objects
+                ]
+            )
+        return cls._cached_dtype
 
     @classmethod
     def empty(cls, length: int) -> "PointsArray":
@@ -184,22 +190,26 @@ class PredictedPointsArray(PointsArray):
             np.dtype: A structured numpy dtype with fields for xy coordinates,
                 score, visible flag, complete flag, and node names.
         """
-        return np.dtype(
-            [
-                ("xy", "<f8", (2,)),  # 64-bit (8-byte) little-endian double, ndim=2
-                ("score", "<f8"),  # 64-bit (8-byte) little-endian double
-                ("visible", "bool"),
-                ("complete", "bool"),
-                (
-                    "name",
-                    "O",
-                ),  # object dtype to store pointers to python string objects
-            ]
-        )
+        # Cache the dtype at the class level for performance
+        # Use cls.__dict__ to check if defined on this class (not inherited)
+        if "_cached_dtype" not in cls.__dict__:
+            cls._cached_dtype = np.dtype(
+                [
+                    ("xy", "<f8", (2,)),  # 64-bit (8-byte) little-endian double, ndim=2
+                    ("score", "<f8"),  # 64-bit (8-byte) little-endian double
+                    ("visible", "bool"),
+                    ("complete", "bool"),
+                    (
+                        "name",
+                        "O",
+                    ),  # object dtype to store pointers to python string objects
+                ]
+            )
+        return cls._cached_dtype
 
     @classmethod
     def from_array(cls, array: np.ndarray) -> "PredictedPointsArray":
-        """Convert an existing array to a PredictedPointsArray with the appropriate dtype.
+        """Convert an existing array to a PredictedPointsArray with appropriate dtype.
 
         Args:
             array: A numpy array to convert. Can be a structured array or a regular
@@ -219,7 +229,8 @@ class PredictedPointsArray(PointsArray):
             - Fourth column (if present) is interpreted as visible flag
             - Fifth column (if present) is interpreted as complete flag
 
-            If visibility is not provided, it is inferred from NaN values in the x coordinate.
+            If visibility is not provided, it is inferred from NaN values in the x
+            coordinate.
         """
         dtype = cls._get_dtype()
 
@@ -328,6 +339,51 @@ class Track:
 
     name: str = ""
 
+    def matches(self, other: "Track", method: str = "name") -> bool:
+        """Check if this track matches another track.
+
+        Args:
+            other: Another track to compare with.
+            method: Matching method - "name" (match by name) or "identity"
+                (match by object identity).
+
+        Returns:
+            True if the tracks match according to the specified method.
+        """
+        if method == "name":
+            return self.name == other.name
+        elif method == "identity":
+            return self is other
+        else:
+            raise ValueError(f"Unknown matching method: {method}")
+
+    def similarity_to(self, other: "Track") -> dict[str, any]:
+        """Calculate similarity metrics with another track.
+
+        Args:
+            other: Another track to compare with.
+
+        Returns:
+            A dictionary with similarity metrics:
+            - 'same_name': Whether the tracks have the same name
+            - 'same_identity': Whether the tracks are the same object
+            - 'name_similarity': Simple string similarity score (0-1)
+        """
+        # Calculate simple string similarity
+        if self.name and other.name:
+            # Simple character overlap similarity
+            common_chars = set(self.name.lower()) & set(other.name.lower())
+            all_chars = set(self.name.lower()) | set(other.name.lower())
+            name_similarity = len(common_chars) / len(all_chars) if all_chars else 0
+        else:
+            name_similarity = 1.0 if self.name == other.name else 0.0
+
+        return {
+            "same_name": self.name == other.name,
+            "same_identity": self is other,
+            "name_similarity": name_similarity,
+        }
+
 
 @attrs.define(auto_attribs=True, slots=True, eq=False)
 class Instance:
@@ -499,7 +555,7 @@ class Instance:
 
     def __getitem__(self, node: Union[int, str, Node]) -> np.ndarray:
         """Return the point associated with a node."""
-        if type(node) != int:
+        if type(node) is not int:
             node = self.skeleton.index(node)
 
         return self.points[node]
@@ -515,7 +571,7 @@ class Instance:
         Notes:
             This sets the point coordinates and marks the point as visible.
         """
-        if type(node) != int:
+        if type(node) is not int:
             node = self.skeleton.index(node)
 
         if len(value) < 2:
@@ -608,6 +664,158 @@ class Instance:
         self.points = new_points
         self.points["name"] = self.skeleton.node_names
 
+    def same_pose_as(self, other: "Instance", tolerance: float = None) -> bool:
+        """Check if this instance has the same pose as another instance.
+
+        Args:
+            other: Another instance to compare with.
+            tolerance: Maximum distance (in pixels) between corresponding points
+                for them to be considered the same. If None (default), uses exact
+                comparison including proper NaN handling.
+
+        Returns:
+            True if the instances have the same pose within tolerance, False otherwise.
+
+        Notes:
+            Two instances are considered to have the same pose if:
+            - They have the same skeleton structure
+            - When tolerance is None: All coordinates match exactly (including NaN)
+            - When tolerance is specified: All visible points are within tolerance
+              distance and NaN patterns match exactly
+        """
+        # Check skeleton compatibility
+        if not self.skeleton.matches(other.skeleton):
+            return False
+
+        if tolerance is None:
+            # Exact comparison using numpy arrays with proper NaN handling
+            return np.array_equal(self.numpy(), other.numpy(), equal_nan=True)
+        else:
+            # Tolerance-based comparison with proper NaN handling
+            self_array = self.numpy()
+            other_array = other.numpy()
+
+            # First, check if NaN patterns match exactly
+            self_nan_mask = np.isnan(self_array)
+            other_nan_mask = np.isnan(other_array)
+            if not np.array_equal(self_nan_mask, other_nan_mask):
+                return False
+
+            # Get mask for non-NaN values
+            non_nan_mask = ~self_nan_mask
+
+            # If all values are NaN, they're considered equal
+            if not non_nan_mask.any():
+                return True
+
+            # Calculate distances only for non-NaN points
+            self_pts = self_array[non_nan_mask]
+            other_pts = other_array[non_nan_mask]
+
+            # Reshape to handle the coordinate pairs properly
+            self_pts = self_pts.reshape(-1, 2)
+            other_pts = other_pts.reshape(-1, 2)
+
+            distances = np.linalg.norm(self_pts - other_pts, axis=1)
+
+            return np.all(distances <= tolerance)
+
+    def same_identity_as(self, other: "Instance") -> bool:
+        """Check if this instance has the same identity (track) as another instance.
+
+        Args:
+            other: Another instance to compare with.
+
+        Returns:
+            True if both instances have the same track identity, False otherwise.
+
+        Notes:
+            Instances have the same identity if they share the same Track object
+            (by identity, not just by name).
+        """
+        if self.track is None or other.track is None:
+            return False
+        return self.track is other.track
+
+    def overlaps_with(self, other: "Instance", iou_threshold: float = 0.5) -> bool:
+        """Check if this instance overlaps with another based on bounding box IoU.
+
+        Args:
+            other: Another instance to compare with.
+            iou_threshold: Minimum IoU (Intersection over Union) value to consider
+                the instances as overlapping.
+
+        Returns:
+            True if the instances overlap above the threshold, False otherwise.
+
+        Notes:
+            Overlap is computed using the bounding boxes of visible points.
+            If either instance has no visible points, they don't overlap.
+        """
+        # Get visible points for both instances
+        self_visible = self.points["visible"]
+        other_visible = other.points["visible"]
+
+        if not self_visible.any() or not other_visible.any():
+            return False
+
+        # Calculate bounding boxes
+        self_pts = self.points["xy"][self_visible]
+        other_pts = other.points["xy"][other_visible]
+
+        self_bbox = np.array(
+            [
+                [np.min(self_pts[:, 0]), np.min(self_pts[:, 1])],  # min x, y
+                [np.max(self_pts[:, 0]), np.max(self_pts[:, 1])],  # max x, y
+            ]
+        )
+
+        other_bbox = np.array(
+            [
+                [np.min(other_pts[:, 0]), np.min(other_pts[:, 1])],
+                [np.max(other_pts[:, 0]), np.max(other_pts[:, 1])],
+            ]
+        )
+
+        # Calculate intersection
+        intersection_min = np.maximum(self_bbox[0], other_bbox[0])
+        intersection_max = np.minimum(self_bbox[1], other_bbox[1])
+
+        if np.any(intersection_min >= intersection_max):
+            # No intersection
+            return False
+
+        intersection_area = np.prod(intersection_max - intersection_min)
+
+        # Calculate union
+        self_area = np.prod(self_bbox[1] - self_bbox[0])
+        other_area = np.prod(other_bbox[1] - other_bbox[0])
+        union_area = self_area + other_area - intersection_area
+
+        # Calculate IoU
+        iou = intersection_area / union_area if union_area > 0 else 0
+
+        return iou >= iou_threshold
+
+    def bounding_box(self) -> Optional[np.ndarray]:
+        """Get the bounding box of visible points.
+
+        Returns:
+            A numpy array of shape (2, 2) with [[min_x, min_y], [max_x, max_y]],
+            or None if there are no visible points.
+        """
+        visible = self.points["visible"]
+        if not visible.any():
+            return None
+
+        pts = self.points["xy"][visible]
+        return np.array(
+            [
+                [np.min(pts[:, 0]), np.min(pts[:, 1])],
+                [np.max(pts[:, 0]), np.max(pts[:, 1])],
+            ]
+        )
+
 
 @attrs.define(eq=False)
 class PredictedInstance(Instance):
@@ -825,12 +1033,13 @@ class PredictedInstance(Instance):
             node: The node to set the point for. Can be an integer index, string name,
                 or Node object.
             value: A tuple or array-like of length 2 or 3 containing (x, y) coordinates
-                and optionally a confidence score. If the score is not provided, it defaults to 1.0.
+                and optionally a confidence score. If the score is not provided, it
+                defaults to 1.0.
 
         Notes:
             This sets the point coordinates, score, and marks the point as visible.
         """
-        if type(node) != int:
+        if type(node) is not int:
             node = self.skeleton.index(node)
 
         if len(value) < 2:
