@@ -5,14 +5,17 @@ a video and its components used in SLEAP.
 """
 
 from __future__ import annotations
-import attrs
-from typing import Tuple, Optional, Optional, Any
-import numpy as np
-from sleap_io.io.video_reading import VideoBackend, MediaVideo, HDF5Video, ImageVideo
-from sleap_io.io.video_writing import VideoWriter
-from sleap_io.io.utils import is_file_accessible
+
 from pathlib import Path
+from typing import Any, Optional, Tuple
+
+import attrs
 import h5py
+import numpy as np
+
+from sleap_io.io.utils import is_file_accessible
+from sleap_io.io.video_reading import HDF5Video, ImageVideo, MediaVideo, VideoBackend
+from sleap_io.io.video_writing import VideoWriter
 
 
 @attrs.define(eq=False)
@@ -48,7 +51,27 @@ class Video:
         two `Video` instances with the same attributes will NOT be considered equal in a
         set or dict.
 
-    See also: VideoBackend
+    Media Video Plugin Support:
+        For media files (mp4, avi, etc.), the following plugins are supported:
+        - "opencv": Uses OpenCV (cv2) for video reading
+        - "FFMPEG": Uses imageio-ffmpeg for video reading
+        - "pyav": Uses PyAV for video reading
+
+        Plugin aliases (case-insensitive):
+        - opencv: "opencv", "cv", "cv2", "ocv"
+        - FFMPEG: "FFMPEG", "ffmpeg", "imageio-ffmpeg", "imageio_ffmpeg"
+        - pyav: "pyav", "av"
+
+        Plugin selection priority:
+        1. Explicitly specified plugin parameter
+        2. Backend metadata plugin value
+        3. Global default (set via sio.set_default_video_plugin)
+        4. Auto-detection based on available packages
+
+    See Also:
+        VideoBackend: The backend interface for reading video data.
+        sleap_io.set_default_video_plugin: Set global default plugin.
+        sleap_io.get_default_video_plugin: Get current default plugin.
     """
 
     filename: str | list[str]
@@ -65,7 +88,7 @@ class Video:
         if self.open_backend and self.backend is None and self.exists():
             try:
                 self.open()
-            except Exception as e:
+            except Exception:
                 # If we can't open the backend, just ignore it for now so we don't
                 # prevent the user from building the Video object entirely.
                 pass
@@ -122,6 +145,9 @@ class Video:
             source_video: The source video object if this is a proxy video. This is
                 present when the video contains an embedded subset of frames from
                 another video.
+            **kwargs: Additional backend-specific arguments passed to
+                VideoBackend.from_filename. See VideoBackend.from_filename for supported
+                arguments.
 
         Returns:
             Video instance with the appropriate backend instantiated.
@@ -155,7 +181,7 @@ class Video:
         """
         try:
             return self.backend.shape
-        except:
+        except Exception:
             if "shape" in self.backend_metadata:
                 return self.backend_metadata["shape"]
             return None
@@ -265,7 +291,7 @@ class Video:
             has_dataset = False
             if (
                 self.backend is not None
-                and type(self.backend) == HDF5Video
+                and type(self.backend) is HDF5Video
                 and self.backend._open_reader is not None
             ):
                 has_dataset = dataset in self.backend._open_reader
@@ -287,6 +313,7 @@ class Video:
         dataset: Optional[str] = None,
         grayscale: Optional[str] = None,
         keep_open: bool = True,
+        plugin: Optional[str] = None,
     ):
         """Open the video backend for reading.
 
@@ -300,6 +327,10 @@ class Video:
                 frames. If False, will close the reader after each call. If True (the
                 default), it will keep the reader open and cache it for subsequent calls
                 which may enhance the performance of reading multiple frames.
+            plugin: Video plugin to use for MediaVideo files. One of "opencv",
+                "FFMPEG", or "pyav". Also accepts aliases (case-insensitive).
+                If not specified, uses the backend metadata, global default,
+                or auto-detection in that order.
 
         Notes:
             This is useful for opening the video backend to read frames and then closing
@@ -330,8 +361,7 @@ class Video:
 
         if not self.exists(dataset=dataset):
             msg = (
-                f"Video does not exist or cannot be opened for reading: "
-                f"{self.filename}"
+                f"Video does not exist or cannot be opened for reading: {self.filename}"
             )
             if dataset is not None:
                 msg += f" (dataset: {dataset})"
@@ -340,12 +370,24 @@ class Video:
         # Close previous backend if open.
         self.close()
 
+        # Handle plugin parameter
+        backend_kwargs = {}
+        if plugin is not None:
+            from sleap_io.io.video_reading import normalize_plugin_name
+
+            plugin = normalize_plugin_name(plugin)
+            self.backend_metadata["plugin"] = plugin
+
+        if "plugin" in self.backend_metadata:
+            backend_kwargs["plugin"] = self.backend_metadata["plugin"]
+
         # Create new backend.
         self.backend = VideoBackend.from_filename(
             self.filename,
             dataset=dataset,
             grayscale=grayscale,
             keep_open=keep_open,
+            **backend_kwargs,
         )
 
     def close(self):
@@ -361,7 +403,7 @@ class Video:
                     self.backend, "grayscale", None
                 )
                 self.backend_metadata["shape"] = getattr(self.backend, "shape", None)
-            except:
+            except Exception:
                 pass
 
             del self.backend
@@ -394,6 +436,204 @@ class Video:
             else:
                 self.close()
 
+    def matches_path(self, other: "Video", strict: bool = False) -> bool:
+        """Check if this video has the same path as another video.
+
+        Args:
+            other: Another video to compare with.
+            strict: If True, require exact path match. If False, consider videos
+                with the same filename (basename) as matching.
+
+        Returns:
+            True if the videos have matching paths, False otherwise.
+        """
+        if isinstance(self.filename, list) and isinstance(other.filename, list):
+            # Both are image sequences
+            if strict:
+                return self.filename == other.filename
+            else:
+                # Compare basenames
+                self_basenames = [Path(f).name for f in self.filename]
+                other_basenames = [Path(f).name for f in other.filename]
+                return self_basenames == other_basenames
+        elif isinstance(self.filename, list) or isinstance(other.filename, list):
+            # One is image sequence, other is single file
+            return False
+        else:
+            # Both are single files
+            if strict:
+                return Path(self.filename).resolve() == Path(other.filename).resolve()
+            else:
+                return Path(self.filename).name == Path(other.filename).name
+
+    def matches_content(self, other: "Video") -> bool:
+        """Check if this video has the same content as another video.
+
+        Args:
+            other: Another video to compare with.
+
+        Returns:
+            True if the videos have the same shape and backend type.
+
+        Notes:
+            This compares metadata like shape and backend type, not actual frame data.
+        """
+        # Compare shapes
+        self_shape = self.shape
+        other_shape = other.shape
+
+        if self_shape != other_shape:
+            return False
+
+        # Compare backend types
+        if self.backend is None and other.backend is None:
+            return True
+        elif self.backend is None or other.backend is None:
+            return False
+
+        return type(self.backend).__name__ == type(other.backend).__name__
+
+    def matches_shape(self, other: "Video") -> bool:
+        """Check if this video has the same shape as another video.
+
+        Args:
+            other: Another video to compare with.
+
+        Returns:
+            True if the videos have the same height, width, and channels.
+
+        Notes:
+            This only compares spatial dimensions, not the number of frames.
+        """
+        # Try to get shape from backend metadata first if shape is not available
+        if self.backend is None and "shape" in self.backend_metadata:
+            self_shape = self.backend_metadata["shape"]
+        else:
+            self_shape = self.shape
+
+        if other.backend is None and "shape" in other.backend_metadata:
+            other_shape = other.backend_metadata["shape"]
+        else:
+            other_shape = other.shape
+
+        # Handle None shapes
+        if self_shape is None or other_shape is None:
+            return False
+
+        # Compare only height, width, channels (not frames)
+        return self_shape[1:] == other_shape[1:]
+
+    def has_overlapping_images(self, other: "Video") -> bool:
+        """Check if this video has overlapping images with another video.
+
+        This method is specifically for ImageVideo backends (image sequences).
+
+        Args:
+            other: Another video to compare with.
+
+        Returns:
+            True if both are ImageVideo instances with overlapping image files.
+            False if either video is not an ImageVideo or no overlap exists.
+
+        Notes:
+            Only works with ImageVideo backends where filename is a list.
+            Compares individual image filenames (basenames only).
+        """
+        # Both must be image sequences
+        if not (isinstance(self.filename, list) and isinstance(other.filename, list)):
+            return False
+
+        # Get basenames for comparison
+        self_basenames = set(Path(f).name for f in self.filename)
+        other_basenames = set(Path(f).name for f in other.filename)
+
+        # Check if there's any overlap
+        return len(self_basenames & other_basenames) > 0
+
+    def deduplicate_with(self, other: "Video") -> "Video":
+        """Create a new video with duplicate images removed.
+
+        This method is specifically for ImageVideo backends (image sequences).
+
+        Args:
+            other: Another video to deduplicate against. Must also be ImageVideo.
+
+        Returns:
+            A new Video object with duplicate images removed from this video,
+            or None if all images were duplicates.
+
+        Raises:
+            ValueError: If either video is not an ImageVideo backend.
+
+        Notes:
+            Only works with ImageVideo backends where filename is a list.
+            Images are considered duplicates if they have the same basename.
+            The returned video contains only images from this video that are
+            not present in the other video.
+        """
+        if not isinstance(self.filename, list):
+            raise ValueError("deduplicate_with only works with ImageVideo backends")
+        if not isinstance(other.filename, list):
+            raise ValueError("Other video must also be ImageVideo backend")
+
+        # Get basenames from other video
+        other_basenames = set(Path(f).name for f in other.filename)
+
+        # Keep only non-duplicate images
+        deduplicated_paths = [
+            f for f in self.filename if Path(f).name not in other_basenames
+        ]
+
+        if not deduplicated_paths:
+            # All images were duplicates
+            return None
+
+        # Create new video with deduplicated images
+        return Video.from_filename(deduplicated_paths, grayscale=self.grayscale)
+
+    def merge_with(self, other: "Video") -> "Video":
+        """Merge another video's images into this one.
+
+        This method is specifically for ImageVideo backends (image sequences).
+
+        Args:
+            other: Another video to merge with. Must also be ImageVideo.
+
+        Returns:
+            A new Video object with unique images from both videos.
+
+        Raises:
+            ValueError: If either video is not an ImageVideo backend.
+
+        Notes:
+            Only works with ImageVideo backends where filename is a list.
+            The merged video contains all unique images from both videos,
+            with automatic deduplication based on image basename.
+        """
+        if not isinstance(self.filename, list):
+            raise ValueError("merge_with only works with ImageVideo backends")
+        if not isinstance(other.filename, list):
+            raise ValueError("Other video must also be ImageVideo backend")
+
+        # Get all unique images (by basename) preserving order
+        seen_basenames = set()
+        merged_paths = []
+
+        for path in self.filename:
+            basename = Path(path).name
+            if basename not in seen_basenames:
+                merged_paths.append(path)
+                seen_basenames.add(basename)
+
+        for path in other.filename:
+            basename = Path(path).name
+            if basename not in seen_basenames:
+                merged_paths.append(path)
+                seen_basenames.add(basename)
+
+        # Create new video with merged images
+        return Video.from_filename(merged_paths, grayscale=self.grayscale)
+
     def save(
         self,
         save_path: str | Path,
@@ -421,3 +661,36 @@ class Video:
 
         new_video = Video.from_filename(save_path, grayscale=self.grayscale)
         return new_video
+
+    def set_video_plugin(self, plugin: str) -> None:
+        """Set the video plugin and reopen the video.
+
+        Args:
+            plugin: Video plugin to use. One of "opencv", "FFMPEG", or "pyav".
+                Also accepts aliases (case-insensitive).
+
+        Raises:
+            ValueError: If the video is not a MediaVideo type.
+
+        Examples:
+            >>> video.set_video_plugin("opencv")
+            >>> video.set_video_plugin("CV2")  # Same as "opencv"
+        """
+        from sleap_io.io.video_reading import MediaVideo, normalize_plugin_name
+
+        if not self.filename.endswith(MediaVideo.EXTS):
+            raise ValueError(f"Cannot set plugin for non-media video: {self.filename}")
+
+        plugin = normalize_plugin_name(plugin)
+
+        # Close current backend if open
+        was_open = self.is_open
+        if was_open:
+            self.close()
+
+        # Update backend metadata
+        self.backend_metadata["plugin"] = plugin
+
+        # Reopen with new plugin if it was open
+        if was_open:
+            self.open()
