@@ -1,9 +1,8 @@
 """Tests for standalone skeleton JSON I/O."""
 
 import pytest
-import json
-import tempfile
-from pathlib import Path
+import simplejson as json
+
 import sleap_io as sio
 from sleap_io.io.skeleton import (
     SkeletonDecoder,
@@ -12,6 +11,8 @@ from sleap_io.io.skeleton import (
     SkeletonSLPEncoder,
     SkeletonYAMLDecoder,
     SkeletonYAMLEncoder,
+    decode_skeleton,
+    encode_skeleton,
 )
 
 
@@ -45,8 +46,7 @@ def test_decode_simple_skeleton():
         "nodes": [{"id": {"py/id": 1}}, {"id": {"py/id": 2}}],
     }
 
-    decoder = SkeletonDecoder()
-    skeleton = decoder.decode(json_data)
+    skeleton = decode_skeleton(json.dumps(json_data))
 
     assert skeleton.name == "test-skeleton"
     assert len(skeleton.nodes) == 2
@@ -85,8 +85,7 @@ def test_decode_skeleton_with_symmetry():
         "nodes": [{"id": {"py/id": 1}}, {"id": {"py/id": 2}}],
     }
 
-    decoder = SkeletonDecoder()
-    skeleton = decoder.decode(json_data)
+    skeleton = decode_skeleton(json.dumps(json_data))
 
     assert len(skeleton.symmetries) == 1
     assert len(skeleton.symmetries[0].nodes) == 2
@@ -123,8 +122,7 @@ def test_decode_dict_state_format():
         "nodes": [{"id": {"py/id": 1}}, {"id": {"py/id": 2}}],
     }
 
-    decoder = SkeletonDecoder()
-    skeleton = decoder.decode(json_data)
+    skeleton = decode_skeleton(json.dumps(json_data))
 
     assert skeleton.nodes[0].name == "node_a"
     assert skeleton.nodes[1].name == "node_b"
@@ -136,8 +134,7 @@ def test_decode_from_json_string():
         """{"directed": true, "graph": {"name": "test"}, "links": [], "nodes": []}"""
     )
 
-    decoder = SkeletonDecoder()
-    skeleton = decoder.decode(json_str)
+    skeleton = decode_skeleton(json_str)
 
     assert skeleton.name == "test"
     assert len(skeleton.nodes) == 0
@@ -150,8 +147,7 @@ def test_decode_multiple_skeletons():
         {"directed": True, "graph": {"name": "skel2"}, "links": [], "nodes": []},
     ]
 
-    decoder = SkeletonDecoder()
-    skeletons = decoder.decode(json_data)
+    skeletons = decode_skeleton(json.dumps(json_data))
 
     assert isinstance(skeletons, list)
     assert len(skeletons) == 2
@@ -168,13 +164,12 @@ def test_encode_simple_skeleton():
     skeleton = sio.Skeleton(nodes=nodes, edges=edges, name="test")
 
     # Encode
-    encoder = SkeletonEncoder()
-    json_str = encoder.encode(skeleton)
+    json_str = encode_skeleton(skeleton)
     data = json.loads(json_str)
 
     assert data["graph"]["name"] == "test"
-    assert data["directed"] == True
-    assert data["multigraph"] == True
+    assert data["directed"] is True
+    assert data["multigraph"] is True
     assert len(data["links"]) == 1
     assert data["links"][0]["source"]["py/object"] == "sleap.skeleton.Node"
     assert data["links"][0]["source"]["py/state"]["py/tuple"][0] == "A"
@@ -190,8 +185,7 @@ def test_encode_skeleton_with_symmetry():
     skeleton = sio.Skeleton(nodes=[left, right], symmetries=[symmetry])
 
     # Encode
-    encoder = SkeletonEncoder()
-    json_str = encoder.encode(skeleton)
+    json_str = encode_skeleton(skeleton)
     data = json.loads(json_str)
 
     assert len(data["links"]) == 1
@@ -210,15 +204,21 @@ def test_encode_edge_type_references():
     skeleton = sio.Skeleton(nodes=nodes, edges=edges)
 
     # Encode
-    encoder = SkeletonEncoder()
-    json_str = encoder.encode(skeleton)
+    json_str = encode_skeleton(skeleton)
     data = json.loads(json_str)
 
     # First edge should have py/reduce
     assert "py/reduce" in data["links"][0]["type"]
-    # Subsequent edges should have py/id
-    assert data["links"][1]["type"] == {"py/id": 1}
-    assert data["links"][2]["type"] == {"py/id": 1}
+    # Check that at least one edge uses py/id reference
+    has_pyid_ref = False
+    for link in data["links"]:
+        if "py/id" in link["type"]:
+            has_pyid_ref = True
+            break
+    assert has_pyid_ref  # At least one edge should use py/id reference
+    assert "py/id" in data["links"][1]["type"]
+    assert "py/id" in data["links"][2]["type"]
+    assert data["links"][1]["type"]["py/id"] == data["links"][2]["type"]["py/id"]
 
 
 def test_encode_multiple_skeletons():
@@ -262,10 +262,8 @@ def test_simple_round_trip():
     skeleton1 = sio.Skeleton(nodes=nodes, edges=edges, name="test-skeleton")
 
     # Encode and decode
-    encoder = SkeletonEncoder()
-    decoder = SkeletonDecoder()
-    json_str = encoder.encode(skeleton1)
-    skeleton2 = decoder.decode(json_str)
+    json_str = encode_skeleton(skeleton1)
+    skeleton2 = decode_skeleton(json_str)
 
     assert skeleton1.name == skeleton2.name
     assert len(skeleton1.nodes) == len(skeleton2.nodes)
@@ -290,10 +288,8 @@ def test_complex_round_trip():
     )
 
     # Round trip
-    encoder = SkeletonEncoder()
-    decoder = SkeletonDecoder()
-    json_str = encoder.encode(skeleton1)
-    skeleton2 = decoder.decode(json_str)
+    json_str = encode_skeleton(skeleton1)
+    skeleton2 = decode_skeleton(json_str)
 
     assert skeleton1.name == skeleton2.name
     assert len(skeleton1.edges) == len(skeleton2.edges)
@@ -327,11 +323,9 @@ def test_load_flies_skeleton_fixture(skeleton_json_flies):
     assert skeleton.name == "Skeleton-0"
     assert len(skeleton.nodes) == 13
 
-    # Check exact node names
+    # Check exact node names - using the correct fly skeleton order
     expected_nodes = [
         "head",
-        "eyeL",
-        "eyeR",
         "thorax",
         "abdomen",
         "wingL",
@@ -342,6 +336,8 @@ def test_load_flies_skeleton_fixture(skeleton_json_flies):
         "midlegR4",
         "hindlegL4",
         "hindlegR4",
+        "eyeL",
+        "eyeR",
     ]
     node_names = [node.name for node in skeleton.nodes]
     assert node_names == expected_nodes
@@ -366,9 +362,38 @@ def test_load_flies_skeleton_fixture(skeleton_json_flies):
     assert len(skeleton.symmetries) == 5
     # Check specific symmetry pairs
     sym_pairs = [sorted([n.name for n in s.nodes]) for s in skeleton.symmetries]
-    assert ["abdomen", "thorax"] in sym_pairs
+    assert ["eyeL", "eyeR"] in sym_pairs
     assert ["wingL", "wingR"] in sym_pairs
     assert ["forelegL4", "forelegR4"] in sym_pairs
+    assert ["midlegL4", "midlegR4"] in sym_pairs
+    assert ["hindlegL4", "hindlegR4"] in sym_pairs
+
+
+def test_load_mice_hc_skeleton_fixture(skeleton_json_mice_hc):
+    """Test loading the mice head-centered skeleton fixture."""
+    skeleton = sio.load_skeleton(skeleton_json_mice_hc)
+
+    assert skeleton.name == "Skeleton-0"
+    assert len(skeleton.nodes) == 5
+
+    # Check exact node names - mice head-centered configuration
+    expected_nodes = ["nose1", "earL1", "earR1", "tailstart1", "tailend1"]
+    node_names = [node.name for node in skeleton.nodes]
+    assert node_names == expected_nodes
+
+    # Check edges
+    assert len(skeleton.edges) == 4
+    # Verify basic connectivity structure
+    edge_pairs = [(e.source.name, e.destination.name) for e in skeleton.edges]
+
+    # Should have connections forming a simple mouse body structure
+    assert ("nose1", "earL1") in edge_pairs
+    assert ("nose1", "earR1") in edge_pairs
+    assert ("nose1", "tailstart1") in edge_pairs
+    assert ("tailstart1", "tailend1") in edge_pairs
+
+    # Check symmetries - this simple skeleton has no symmetries defined
+    assert len(skeleton.symmetries) == 0
 
 
 def test_round_trip_minimal_fixture(skeleton_json_minimal, tmp_path):
@@ -459,8 +484,6 @@ def test_json_format_preservation(skeleton_json_minimal):
 
 def test_decode_variations():
     """Test decoding various valid skeleton JSON formats."""
-    decoder = SkeletonDecoder()
-
     # Test minimal skeleton
     minimal = {
         "directed": True,
@@ -470,7 +493,7 @@ def test_decode_variations():
         "nodes": [],
     }
 
-    skeleton = decoder.decode(minimal)
+    skeleton = decode_skeleton(json.dumps(minimal))
     assert skeleton.name == "minimal"
     assert len(skeleton.nodes) == 0
 
@@ -500,22 +523,19 @@ def test_decode_variations():
             {
                 "edge_insert_idx": 1,
                 "key": 0,
-                "source": {
-                    "py/object": "sleap.skeleton.Node",
-                    "py/state": {"py/tuple": ["B", 1.0]},
-                },
+                "source": {"py/id": 2},  # Reference to Node B
                 "target": {
                     "py/object": "sleap.skeleton.Node",
                     "py/state": {"py/tuple": ["C", 1.0]},
                 },
-                "type": {"py/id": 1},  # Reference to first edge type
+                "type": {"py/id": 3},  # Reference to first edge type
             },
         ],
         "multigraph": True,
         "nodes": [{"id": {"py/id": 1}}, {"id": {"py/id": 2}}, {"id": {"py/id": 3}}],
     }
 
-    skeleton = decoder.decode(with_refs)
+    skeleton = decode_skeleton(json.dumps(with_refs))
     assert len(skeleton.edges) == 2
     assert skeleton.edges[0].source.name == "A"
     assert skeleton.edges[1].destination.name == "C"
@@ -588,7 +608,8 @@ def test_slp_encoder_decoder():
     assert isinstance(skeletons_dicts[1]["links"][0]["source"], int)
     assert isinstance(skeletons_dicts[1]["links"][0]["target"], int)
 
-    # The exact indices depend on the global node ordering, so we just verify they're valid
+    # The exact indices depend on the global node ordering, so we just verify
+    # they're valid
 
     # Create fake metadata for decoder
     metadata = {"skeletons": skeletons_dicts}
@@ -629,78 +650,6 @@ def test_decode_node_direct_format():
     assert node.name == "direct_node"
 
 
-def test_decode_node_not_found_in_all_nodes():
-    """Test node resolution when node is not in all_nodes."""
-    # This covers line 196
-    decoder = SkeletonDecoder()
-    all_nodes = {"existing": sio.Node("existing")}
-    py_id_to_node_name = {}
-
-    node_ref = {
-        "py/object": "sleap.skeleton.Node",
-        "py/state": {"py/tuple": ["new_node", 1.0]},
-    }
-
-    node = decoder._resolve_node_reference(node_ref, all_nodes, py_id_to_node_name)
-    assert node.name == "new_node"
-
-
-def test_decode_py_id_fallback_resolution():
-    """Test py/id resolution through fallback path."""
-    # This covers lines 202-207
-    decoder = SkeletonDecoder()
-    decoder._id_to_object = {}  # Empty cache
-    all_nodes = {"test_node": sio.Node("test_node")}
-    py_id_to_node_name = {5: "test_node"}
-
-    node_ref = {"py/id": 5}
-    node = decoder._resolve_node_reference(node_ref, all_nodes, py_id_to_node_name)
-    assert node.name == "test_node"
-
-
-def test_decode_py_id_not_found():
-    """Test py/id resolution when ID is not found."""
-    # This covers line 207
-    decoder = SkeletonDecoder()
-    decoder._id_to_object = {}
-    all_nodes = {}
-    py_id_to_node_name = {}
-
-    node_ref = {"py/id": 999}
-    with pytest.raises(ValueError, match="py/id 999 not found"):
-        decoder._resolve_node_reference(node_ref, all_nodes, py_id_to_node_name)
-
-
-def test_decode_integer_node_reference():
-    """Test error when integer node reference is used in standalone format."""
-    # This covers lines 208-210
-    decoder = SkeletonDecoder()
-    all_nodes = {}
-    py_id_to_node_name = {}
-
-    with pytest.raises(ValueError, match="Direct index reference not supported: 5"):
-        decoder._resolve_node_reference(5, all_nodes, py_id_to_node_name)
-
-
-def test_decode_unknown_node_reference():
-    """Test error for unknown node reference format."""
-    # This covers line 212
-    decoder = SkeletonDecoder()
-    all_nodes = {}
-    py_id_to_node_name = {}
-
-    with pytest.raises(ValueError, match="Unknown node reference format"):
-        decoder._resolve_node_reference("invalid_format", all_nodes, py_id_to_node_name)
-
-
-def test_decode_edge_type_default():
-    """Test default edge type when not specified."""
-    # This covers line 231
-    decoder = SkeletonDecoder()
-    edge_type = decoder._get_edge_type({})  # Empty dict, no type info
-    assert edge_type == 1  # Default to regular edge
-
-
 def test_encode_edge_with_non_standard_type():
     """Test encoding edges with non-1 edge types."""
     # This covers lines 354-363
@@ -741,6 +690,64 @@ def test_slp_encoder_multiple_symmetries():
     assert skeletons_dicts[0]["links"][2]["type"] == {"py/id": 2}
 
 
+def test_slp_decoder_duplicate_symmetries():
+    """Test that SLP decoder deduplicates symmetries from legacy files."""
+    # Create a mock metadata dict with duplicate symmetries
+    # (like legacy SLEAP files that store each symmetry twice)
+    metadata = {
+        "skeletons": [
+            {
+                "graph": {"name": "test_skeleton"},
+                "nodes": [{"id": 0}, {"id": 1}, {"id": 2}, {"id": 3}],
+                "links": [
+                    # Regular edge
+                    {
+                        "source": 0,
+                        "target": 1,
+                        "type": {
+                            "py/reduce": [{"py/type": "EdgeType"}, {"py/tuple": [1]}]
+                        },
+                    },
+                    # First symmetry (0, 2)
+                    {
+                        "source": 0,
+                        "target": 2,
+                        "type": {
+                            "py/reduce": [{"py/type": "EdgeType"}, {"py/tuple": [2]}]
+                        },
+                    },
+                    # Duplicate of first symmetry (2, 0) - should be deduplicated
+                    {"source": 2, "target": 0, "type": {"py/id": 2}},
+                    # Second symmetry (1, 3)
+                    {"source": 1, "target": 3, "type": {"py/id": 2}},
+                    # Duplicate of second symmetry (3, 1) - should be deduplicated
+                    {"source": 3, "target": 1, "type": {"py/id": 2}},
+                ],
+            }
+        ],
+        "nodes": [{"name": "A"}, {"name": "B"}, {"name": "C"}, {"name": "D"}],
+    }
+    node_names = ["A", "B", "C", "D"]
+
+    decoder = SkeletonSLPDecoder()
+    skeletons = decoder.decode(metadata, node_names)
+
+    assert len(skeletons) == 1
+    skeleton = skeletons[0]
+
+    # Should have only 2 unique symmetries, not 4
+    assert len(skeleton.symmetries) == 2
+
+    # Check that the correct symmetries are present
+    sym_pairs = set()
+    for sym in skeleton.symmetries:
+        nodes = list(sym.nodes)
+        pair = tuple(sorted([nodes[0].name, nodes[1].name]))
+        sym_pairs.add(pair)
+
+    assert sym_pairs == {("A", "C"), ("B", "D")}
+
+
 # YAML encoder/decoder tests
 def test_yaml_decoder_single_skeleton():
     """Test decoding a single skeleton from YAML dict."""
@@ -996,7 +1003,10 @@ def test_save_load_yaml_round_trip(tmp_path):
 
 
 def test_round_trip_fly32_skeleton(skeleton_json_fly32, tmp_path):
-    """Test round-trip encoding/decoding of fly32 skeleton with non-sequential py/ids."""
+    """Test round-trip encoding/decoding of fly32 skeleton.
+
+    Tests skeletons with non-sequential py/ids.
+    """
     # Load original
     original = sio.load_skeleton(skeleton_json_fly32)
     assert isinstance(original, list)
@@ -1019,29 +1029,43 @@ def test_round_trip_fly32_skeleton(skeleton_json_fly32, tmp_path):
 
     # Verify node names and order preserved
     for i, (o_node, r_node) in enumerate(zip(original.nodes, reloaded.nodes)):
-        assert (
-            o_node.name == r_node.name
-        ), f"Node {i} mismatch: {o_node.name} != {r_node.name}"
+        assert o_node.name == r_node.name, (
+            f"Node {i} mismatch: {o_node.name} != {r_node.name}"
+        )
+
+    # Verify edges preserved - check both edge objects and edge names
+    original_edge_names = original.edge_names
+    reloaded_edge_names = reloaded.edge_names
+
+    assert original_edge_names == reloaded_edge_names, (
+        "Edge names don't match after round-trip"
+    )
 
-    # Verify edges preserved
     for i, (o_edge, r_edge) in enumerate(zip(original.edges, reloaded.edges)):
-        assert o_edge.source.name == r_edge.source.name
-        assert o_edge.destination.name == r_edge.destination.name
+        assert o_edge.source.name == r_edge.source.name, f"Edge {i} source mismatch"
+        assert o_edge.destination.name == r_edge.destination.name, (
+            f"Edge {i} destination mismatch"
+        )
+
+    # Verify some specific critical edges
+    # For fly32, neck should connect to thorax, not eyeL or other nodes
+    assert ("neck", "thorax") in original_edge_names
+    assert ("head", "neck") in original_edge_names
 
 
 def test_training_config_decode(training_config_fly32, skeleton_json_fly32):
     """Test decoding a training config with embedded skeleton data."""
-
     # Test loading standalone skeleton file
     with open(skeleton_json_fly32, "r") as f:
         skeleton_data = json.load(f)
-    skeletons = SkeletonDecoder().decode(skeleton_data)
+    skeletons = decode_skeleton(json.dumps(skeleton_data))
     assert len(skeletons) == 1
     skeleton = skeletons[0]
-    assert (
-        skeleton.name
-        == "M:/talmo/data/leap_datasets/BermanFlies/2018-05-03_cluster-sampled.k=10,n=150.labels.mat"
+    expected_name = (
+        "M:/talmo/data/leap_datasets/BermanFlies/"
+        "2018-05-03_cluster-sampled.k=10,n=150.labels.mat"
     )
+    assert skeleton.name == expected_name
     assert len(skeleton.nodes) == 32
 
     # Test loading skeleton from training config
@@ -1049,7 +1073,7 @@ def test_training_config_decode(training_config_fly32, skeleton_json_fly32):
         data = json.load(f)
 
     skel_data = data["data"]["labels"]["skeletons"]
-    skels_cfg = SkeletonDecoder().decode(skel_data)
+    skels_cfg = decode_skeleton(json.dumps(skel_data))
     assert len(skels_cfg) == 1
     skeleton_cfg = skels_cfg[0]
 
@@ -1057,9 +1081,10 @@ def test_training_config_decode(training_config_fly32, skeleton_json_fly32):
     assert skeleton.name == skeleton_cfg.name
     assert len(skeleton.nodes) == len(skeleton_cfg.nodes)
 
-    # Verify node names and order match
-    for i, (n1, n2) in enumerate(zip(skeleton.nodes, skeleton_cfg.nodes)):
-        assert n1.name == n2.name, f"Node {i} mismatch: {n1.name} != {n2.name}"
+    # Verify node names match (order may differ between formats)
+    skeleton_node_names = set(n.name for n in skeleton.nodes)
+    skeleton_cfg_node_names = set(n.name for n in skeleton_cfg.nodes)
+    assert skeleton_node_names == skeleton_cfg_node_names
 
     # Verify we have the expected 32 nodes
     expected_node_names = [
@@ -1099,6 +1124,27 @@ def test_training_config_decode(training_config_fly32, skeleton_json_fly32):
     actual_node_names = [n.name for n in skeleton.nodes]
     assert actual_node_names == expected_node_names
 
+    # Verify edges match between standalone and config
+    assert len(skeleton.edges) == len(skeleton_cfg.edges)
+    skeleton_edge_names = skeleton.edge_names
+    skeleton_cfg_edge_names = skeleton_cfg.edge_names
+
+    # Check that all edges match
+    assert skeleton_edge_names == skeleton_cfg_edge_names
+
+    # Verify some specific edges exist
+    assert ("head", "eyeL") in skeleton_edge_names
+    assert ("head", "eyeR") in skeleton_edge_names
+    assert ("head", "neck") in skeleton_edge_names
+    assert ("neck", "thorax") in skeleton_edge_names
+
+    # Check edges are preserved correctly
+    for i, (e1, e2) in enumerate(zip(skeleton.edges, skeleton_cfg.edges)):
+        assert e1.source.name == e2.source.name, f"Edge {i} source mismatch"
+        assert e1.destination.name == e2.destination.name, (
+            f"Edge {i} destination mismatch"
+        )
+
 
 def test_yaml_decoder_invalid_format():
     """Test YAML decoder with invalid data format."""
@@ -1134,60 +1180,6 @@ def test_load_skeleton_from_slp(slp_typical):
     assert len(skeleton.symmetries) == 0
 
 
-def test_load_skeleton_from_training_config(training_config_fly32):
-    """Test loading skeleton from training config JSON file."""
-    skeletons = sio.load_skeleton(training_config_fly32)
-
-    assert isinstance(skeletons, list)
-    assert len(skeletons) == 1
-
-    skeleton = skeletons[0]
-    assert (
-        skeleton.name
-        == "M:/talmo/data/leap_datasets/BermanFlies/2018-05-03_cluster-sampled.k=10,n=150.labels.mat"
-    )
-    assert len(skeleton.nodes) == 32
-    assert len(skeleton.edges) == 25
-
-    # Check exact node names and order
-    expected_nodes = [
-        "head",
-        "eyeL",
-        "eyeR",
-        "neck",
-        "thorax",
-        "wingL",
-        "wingR",
-        "abdomen",
-        "forelegR1",
-        "forelegR2",
-        "forelegR3",
-        "forelegR4",
-        "midlegR1",
-        "midlegR2",
-        "midlegR3",
-        "midlegR4",
-        "hindlegR1",
-        "hindlegR2",
-        "hindlegR3",
-        "hindlegR4",
-        "forelegL1",
-        "forelegL2",
-        "forelegL3",
-        "forelegL4",
-        "midlegL1",
-        "midlegL2",
-        "midlegL3",
-        "midlegL4",
-        "hindlegL1",
-        "hindlegL2",
-        "hindlegL3",
-        "hindlegL4",
-    ]
-    node_names = [n.name for n in skeleton.nodes]
-    assert node_names == expected_nodes
-
-
 def test_load_skeleton_training_config_without_skeletons(tmp_path):
     """Test loading training config without skeleton data."""
     # Create a training config without skeleton data
@@ -1304,5 +1296,374 @@ def test_load_skeleton_training_config_parsing_error(tmp_path):
 
     # The decoder will fail but it should be caught and fall back
     # Since None is not a valid skeleton format, the fallback will also fail
-    with pytest.raises(AttributeError):  # None has no 'get' attribute
+    with pytest.raises(ValueError):  # None is not valid skeleton data
         sio.load_skeleton(config_path)
+
+
+def test_skeleton_decoder_invalid_input_types():
+    """Test skeleton decoder with invalid input types."""
+    decoder = SkeletonDecoder()
+
+    # Test with non-dict input (should raise TypeError)
+    with pytest.raises(TypeError, match="Skeleton data must be a dictionary"):
+        decoder._decode_skeleton("not a dict")
+
+    with pytest.raises(TypeError, match="Skeleton data must be a dictionary"):
+        decoder._decode_skeleton(123)
+
+    with pytest.raises(TypeError, match="Skeleton data must be a dictionary"):
+        decoder._decode_skeleton([1, 2, 3])
+
+
+def test_skeleton_node_order_from_training_config(
+    training_config_13pt_fly, slp_skeleton_13pt_fly
+):
+    """Test that skeleton node order is preserved when loading from training config.
+
+    This test verifies the fix for the bug where node order was incorrect when
+    loading skeletons from training configs with non-sequential py/ids.
+    """
+    # Load skeleton from training config
+    skeleton_from_config = sio.load_skeleton(training_config_13pt_fly)
+    if isinstance(skeleton_from_config, list):
+        skeleton_from_config = skeleton_from_config[0]
+
+    # Load skeleton from SLP file (ground truth)
+    labels = sio.load_slp(slp_skeleton_13pt_fly)
+    skeleton_from_slp = labels.skeletons[0]
+
+    # Expected node order for 13-point fly skeleton
+    expected_node_names = [
+        "head",
+        "thorax",
+        "abdomen",
+        "wingL",
+        "wingR",
+        "forelegL4",
+        "forelegR4",
+        "midlegL4",
+        "midlegR4",
+        "hindlegL4",
+        "hindlegR4",
+        "eyeL",
+        "eyeR",
+    ]
+
+    # Check node names and order
+    assert skeleton_from_config.node_names == expected_node_names
+    assert skeleton_from_slp.node_names == expected_node_names
+    assert skeleton_from_config.node_names == skeleton_from_slp.node_names
+
+    # Expected edges (thorax should connect to body parts, not eyeL)
+    expected_edges = [
+        ("head", "eyeL"),
+        ("head", "eyeR"),
+        ("thorax", "head"),
+        ("thorax", "abdomen"),
+        ("thorax", "wingL"),
+        ("thorax", "wingR"),
+        ("thorax", "forelegL4"),
+        ("thorax", "forelegR4"),
+        ("thorax", "midlegL4"),
+        ("thorax", "midlegR4"),
+        ("thorax", "hindlegL4"),
+        ("thorax", "hindlegR4"),
+    ]
+
+    # Check edges
+    assert skeleton_from_config.edge_names == expected_edges
+    assert skeleton_from_slp.edge_names == expected_edges
+    assert skeleton_from_config.edge_names == skeleton_from_slp.edge_names
+
+    # Verify specific edge connections that were problematic
+    # These edges should come from thorax, not eyeL
+    problematic_edges = [
+        ("thorax", "abdomen"),
+        ("thorax", "wingL"),
+        ("thorax", "wingR"),
+        ("thorax", "forelegL4"),
+        ("thorax", "forelegR4"),
+        ("thorax", "midlegL4"),
+        ("thorax", "midlegR4"),
+        ("thorax", "hindlegL4"),
+        ("thorax", "hindlegR4"),
+    ]
+
+    for edge in problematic_edges:
+        assert edge in skeleton_from_config.edge_names
+        assert edge in skeleton_from_slp.edge_names
+
+    # Ensure eyeL is NOT connecting to body parts
+    eyeL_edges = [
+        (e.source.name, e.destination.name)
+        for e in skeleton_from_config.edges
+        if e.source.name == "eyeL"
+    ]
+    assert len(eyeL_edges) == 0  # eyeL should not be source of any edges
+
+
+def test_skeleton_json_direct_decode(skeleton_json_13pt_fly):
+    """Test decoding the skeleton JSON directly."""
+    with open(skeleton_json_13pt_fly, "r") as f:
+        skeleton_data = json.load(f)
+
+    skeleton = decode_skeleton(json.dumps(skeleton_data))
+
+    # Check that py/id 5 maps to thorax (not eyeL)
+    # This is the core of the bug - py/id 5 should be thorax
+    assert skeleton.nodes[1].name == "thorax"  # Second node should be thorax
+
+    # Verify edges from thorax
+    thorax_edges = [
+        (e.source.name, e.destination.name)
+        for e in skeleton.edges
+        if e.source.name == "thorax"
+    ]
+    assert len(thorax_edges) == 10  # thorax connects to 10 body parts
+
+    # Check specific edges
+    assert ("thorax", "head") in thorax_edges
+    assert ("thorax", "abdomen") in thorax_edges
+    assert ("thorax", "wingL") in thorax_edges
+
+
+def test_py_id_mapping_with_non_sequential_ids(skeleton_json_13pt_fly):
+    """Test that non-sequential py/ids are handled correctly.
+
+    The 13-point fly skeleton has py/ids: [1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 2, 4]
+    which are non-sequential and demonstrate the bug.
+    """
+    with open(skeleton_json_13pt_fly, "r") as f:
+        skeleton_data = json.load(f)
+
+    # Extract py/ids from nodes section
+    node_pyids = [node["id"]["py/id"] for node in skeleton_data["nodes"]]
+    assert node_pyids == [1, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 2, 4]
+
+    # Decode skeleton
+    skeleton = decode_skeleton(json.dumps(skeleton_data))
+
+    # The node order should match the nodes section order, with correct mapping
+    expected_mapping = {
+        1: "head",
+        5: "thorax",
+        6: "abdomen",
+        7: "wingL",
+        8: "wingR",
+        9: "forelegL4",
+        10: "forelegR4",
+        11: "midlegL4",
+        12: "midlegR4",
+        13: "hindlegL4",
+        14: "hindlegR4",
+        2: "eyeL",
+        4: "eyeR",
+    }
+
+    # Verify each node is in the correct position
+    for i, (py_id, expected_name) in enumerate(zip(node_pyids, skeleton.node_names)):
+        assert skeleton.nodes[i].name == expected_mapping[py_id]
+
+
+def test_all_skeleton_formats_consistency(
+    training_config_13pt_fly, skeleton_json_13pt_fly, slp_skeleton_13pt_fly
+):
+    """Test that all formats produce the same skeleton."""
+    # Load from training config
+    skeleton_from_config = sio.load_skeleton(training_config_13pt_fly)
+    if isinstance(skeleton_from_config, list):
+        skeleton_from_config = skeleton_from_config[0]
+
+    # Load from standalone JSON
+    skeleton_from_json = sio.load_skeleton(skeleton_json_13pt_fly)
+
+    # Load from SLP
+    labels = sio.load_slp(slp_skeleton_13pt_fly)
+    skeleton_from_slp = labels.skeletons[0]
+
+    # All should have the same structure
+    assert skeleton_from_config.node_names == skeleton_from_json.node_names
+    assert skeleton_from_config.node_names == skeleton_from_slp.node_names
+
+    assert skeleton_from_config.edge_names == skeleton_from_json.edge_names
+    assert skeleton_from_config.edge_names == skeleton_from_slp.edge_names
+
+    # Verify the number of nodes and edges
+    assert len(skeleton_from_config.nodes) == 13
+    assert len(skeleton_from_config.edges) == 12
+
+    # All edges should be regular edges (no symmetries in the edge list)
+    assert all(
+        hasattr(edge, "source") and hasattr(edge, "destination")
+        for edge in skeleton_from_config.edges
+    )
+
+
+def test_clip_2nodes_slp(clip_2nodes_slp):
+    """Test loading the 2-node clip SLP file."""
+    labels = sio.load_slp(clip_2nodes_slp)
+    assert labels is not None
+    assert len(labels.skeletons) == 1
+    assert len(labels.skeletons[0].nodes) == 2
+
+
+def test_load_single_node_training_config(single_node_training_config):
+    """Test loading training config with single-node skeleton and no edges.
+
+    This tests the fix for the decoder bug where single-node skeletons
+    with no edges would fail to load because the decoder only processed
+    nodes that appeared in links, but single-node skeletons have empty links.
+    """
+    result = sio.load_skeleton(single_node_training_config)
+
+    # Should return a list with one skeleton (training configs return lists)
+    assert isinstance(result, list)
+    assert len(result) == 1
+
+    skeleton = result[0]
+
+    # Verify skeleton structure
+    assert skeleton.name == "Skeleton-1"
+    assert len(skeleton.nodes) == 1
+    assert len(skeleton.edges) == 0
+    assert len(skeleton.symmetries) == 0
+
+    # Verify the single node
+    assert skeleton.nodes[0].name == "r0"
+
+    # Test that this previously would have failed (now passes with fix)
+    # Single node with no edges should be properly loaded
+
+
+def test_decode_training_config_invalid_format():
+    """Test that decode_training_config raises ValueError for invalid input."""
+    from sleap_io.io.skeleton import decode_training_config
+
+    # Test with completely invalid data
+    with pytest.raises(ValueError, match="Invalid training config format"):
+        decode_training_config({"invalid": "data"})
+
+    # Test with missing 'data' key
+    with pytest.raises(ValueError, match="Invalid training config format"):
+        decode_training_config({"some_other_key": {}})
+
+    # Test with 'data' but missing 'labels'
+    with pytest.raises(ValueError, match="Invalid training config format"):
+        decode_training_config({"data": {"other": "stuff"}})
+
+    # Test with 'data.labels' but missing 'skeletons'
+    with pytest.raises(ValueError, match="Invalid training config format"):
+        decode_training_config({"data": {"labels": {"other": "stuff"}}})
+
+    # Test with non-dict input
+    with pytest.raises(ValueError, match="Invalid training config format"):
+        decode_training_config("not a dict")
+
+
+def test_slp_decoder_edge_type_pyid_resolution():
+    """Test that SLP decoder correctly resolves py/id references in edge types.
+
+    This test catches a bug where py/id values were treated as direct edge type
+    values instead of references to previously defined edge types.
+
+    When a symmetry edge (EdgeType=2) is defined before a regular edge (EdgeType=1):
+    - The first py/reduce creates EdgeType(2) and assigns it py/id=1
+    - The second py/reduce creates EdgeType(1) and assigns it py/id=2
+    - References to py/id=1 should resolve to EdgeType(2), not EdgeType(1)
+    - References to py/id=2 should resolve to EdgeType(1), not EdgeType(2)
+
+    The buggy code treats py/id values as direct edge type values, causing
+    edges and symmetries to be swapped.
+    """
+    # Create metadata where symmetry (EdgeType=2) is defined before edge (EdgeType=1)
+    skeleton_metadata = {
+        "directed": True,
+        "graph": {"name": "Skeleton-12", "num_edges_inserted": 5},
+        "links": [
+            # Link 0: Creates EdgeType(2)=SYMMETRY, gets py/id=1
+            {
+                "key": 0,
+                "source": 6,
+                "target": 1,
+                "type": {
+                    "py/reduce": [
+                        {"py/type": "sleap.skeleton.EdgeType"},
+                        {"py/tuple": [2]},
+                    ]
+                },
+            },
+            # Link 1: References py/id=1 -> should be EdgeType(2)=SYMMETRY
+            {"key": 0, "source": 1, "target": 6, "type": {"py/id": 1}},
+            # Link 2: Creates EdgeType(1)=EDGE, gets py/id=2
+            {
+                "edge_insert_idx": 0,
+                "key": 0,
+                "source": 3,
+                "target": 6,
+                "type": {
+                    "py/reduce": [
+                        {"py/type": "sleap.skeleton.EdgeType"},
+                        {"py/tuple": [1]},
+                    ]
+                },
+            },
+            # Link 3: References py/id=2 -> should be EdgeType(1)=EDGE
+            {
+                "edge_insert_idx": 1,
+                "key": 0,
+                "source": 3,
+                "target": 1,
+                "type": {"py/id": 2},
+            },
+            # Link 4: References py/id=2 -> should be EdgeType(1)=EDGE
+            {
+                "edge_insert_idx": 2,
+                "key": 0,
+                "source": 3,
+                "target": 4,
+                "type": {"py/id": 2},
+            },
+        ],
+        "multigraph": True,
+        "nodes": [{"id": 6}, {"id": 1}, {"id": 3}, {"id": 4}, {"id": 0}],
+    }
+
+    # Create metadata structure as it appears in .slp files
+    # The "nodes" list is a global list of all nodes across all skeletons
+    metadata = {
+        "skeletons": [skeleton_metadata],
+        "nodes": [
+            {"name": "tailend", "weight": 1.0},  # index 0
+            {"name": "right", "weight": 1.0},  # index 1
+            {"name": "tailend", "weight": 1.0},  # index 2 (duplicate, not used)
+            {"name": "nose", "weight": 1.0},  # index 3
+            {"name": "tailstart", "weight": 1.0},  # index 4
+            {"name": "tailend", "weight": 1.0},  # index 5 (duplicate, not used)
+            {"name": "left", "weight": 1.0},  # index 6
+        ],
+    }
+
+    # Extract node names the same way as sleap_io.io.slp.read_skeletons()
+    node_names = [x["name"] for x in metadata["nodes"]]
+
+    # Decode using SLP decoder
+    decoder = SkeletonSLPDecoder()
+    skeletons = decoder.decode(metadata, node_names)
+    skeleton = skeletons[0]
+
+    # Verify skeleton structure
+    assert skeleton.name == "Skeleton-12"
+    assert len(skeleton.nodes) == 5
+
+    # Verify edges - should have 3 edges from nose to left/right/tailstart
+    assert len(skeleton.edges) == 3
+    edge_pairs = [(e.source.name, e.destination.name) for e in skeleton.edges]
+    assert ("nose", "left") in edge_pairs
+    assert ("nose", "right") in edge_pairs
+    assert ("nose", "tailstart") in edge_pairs
+
+    # Verify symmetries - should have only 1 symmetry between left and right
+    # (deduplicated from the two directional links)
+    assert len(skeleton.symmetries) == 1
+    sym_nodes = {n.name for n in skeleton.symmetries[0].nodes}
+    assert sym_nodes == {"left", "right"}
