@@ -2,111 +2,131 @@
 
 from __future__ import annotations
 
+import shutil
+import sys
+from pathlib import Path
+from unittest import mock
+
+import h5py
+import numpy as np
+import pytest
+import simplejson as json
+
 from sleap_io import (
-    Video,
-    Skeleton,
-    Edge,
-    Node,
-    Instance,
-    LabeledFrame,
-    Track,
-    PredictedInstance,
-    Labels,
-    SuggestionFrame,
     Camera,
     CameraGroup,
     FrameGroup,
+    Instance,
     InstanceGroup,
+    LabeledFrame,
+    Labels,
+    LabelsSet,
+    Node,
+    PredictedInstance,
     RecordingSession,
+    Skeleton,
+    SuggestionFrame,
+    Track,
+    Video,
+    get_default_image_plugin,
     load_file,
+    load_slp,
     save_file,
+    save_slp,
+    set_default_image_plugin,
 )
 from sleap_io.io.slp import (
-    read_videos,
-    write_videos,
-    read_tracks,
-    write_tracks,
-    read_instances,
-    read_metadata,
-    read_skeletons,
-    serialize_skeletons,
-    video_to_dict,
-    write_metadata,
-    read_points,
-    read_pred_points,
-    read_instances,
-    write_lfs,
-    read_labels,
-    write_labels,
-    read_suggestions,
-    write_suggestions,
+    camera_group_to_dict,
+    camera_to_dict,
+    embed_frames,
+    embed_videos,
+    frame_group_to_dict,
+    instance_group_to_dict,
     make_camera,
     make_camera_group,
     make_frame_group,
     make_instance_group,
     make_session,
-    camera_to_dict,
-    camera_group_to_dict,
-    frame_group_to_dict,
-    instance_group_to_dict,
-    session_to_dict,
-    read_sessions,
-    write_sessions,
-    embed_frames,
-    embed_videos,
-    process_and_embed_frames,
     prepare_frames_to_embed,
+    process_and_embed_frames,
+    read_instances,
+    read_labels,
+    read_labels_set,
+    read_metadata,
+    read_points,
+    read_pred_points,
+    read_sessions,
+    read_skeletons,
+    read_suggestions,
+    read_tracks,
+    read_videos,
+    session_to_dict,
+    video_to_dict,
+    write_labels,
+    write_lfs,
+    write_metadata,
+    write_sessions,
+    write_suggestions,
+    write_tracks,
+    write_videos,
 )
 from sleap_io.io.utils import read_hdf5_attrs, read_hdf5_dataset
-from sleap_io.io.main import save_slp, save_file, load_slp
-import numpy as np
-import simplejson as json
-import pytest
-from pathlib import Path
-import shutil
-from sleap_io.io.video_reading import ImageVideo, HDF5Video, MediaVideo
-import sys
-import h5py
-from unittest import mock
-from tqdm import tqdm
+from sleap_io.io.video_reading import HDF5Video, ImageVideo, MediaVideo
 
 
 def test_read_labels(slp_typical, slp_simple_skel, slp_minimal):
     """Test `read_labels` can read different types of .slp files."""
     labels = read_labels(slp_typical)
-    assert type(labels) == Labels
+    assert type(labels) is Labels
 
     labels = read_labels(slp_simple_skel)
-    assert type(labels) == Labels
+    assert type(labels) is Labels
 
     labels = read_labels(slp_minimal)
-    assert type(labels) == Labels
+    assert type(labels) is Labels
 
 
 def test_load_slp_with_provenance(slp_predictions_with_provenance):
     labels = read_labels(slp_predictions_with_provenance)
     provenance = labels.provenance
-    assert type(provenance) == dict
+    assert type(provenance) is dict
     assert provenance["sleap_version"] == "1.2.7"
 
 
+def test_legacy_coordinate_system(slp_legacy_grid_labels):
+    """Test that legacy SLP files with FORMAT_ID = 1.0 have correct coordinates.
+
+    Legacy files use a coordinate system where the top-left of the pixel is at (0, 0).
+    Newer files (FORMAT_ID >= 1.1) use (-0.5, -0.5) for the top-left of the pixel,
+    with the pixel center at the origin.
+    """
+    labels = read_labels(slp_legacy_grid_labels)
+
+    # Get the first instance from the first frame
+    inst = labels[0][0]
+
+    # Check that the coordinates match the expected legacy coordinate system
+    # These coordinates should reflect the adjustment from pixel corners to centers
+    np.testing.assert_array_equal(inst.numpy(), [[-1, -1], [-0.5, -0.5], [-1, 0]])
+
+
 def test_read_instances_from_predicted(slp_real_data):
     labels = read_labels(slp_real_data)
 
     lf = labels.find(video=labels.video, frame_idx=220)[0]
     assert len(lf) == 3
-    assert type(lf.instances[0]) == PredictedInstance
-    assert type(lf.instances[1]) == PredictedInstance
-    assert type(lf.instances[2]) == Instance
+    assert type(lf.instances[0]) is PredictedInstance
+    assert type(lf.instances[1]) is PredictedInstance
+    assert type(lf.instances[2]) is Instance
     assert lf.instances[2].from_predicted == lf.instances[1]
     assert lf.unused_predictions == [lf.instances[0]]
 
     lf = labels.find(video=labels.video, frame_idx=770)[0]
     assert len(lf) == 4
-    assert type(lf.instances[0]) == PredictedInstance
-    assert type(lf.instances[1]) == PredictedInstance
-    assert type(lf.instances[2]) == Instance
-    assert type(lf.instances[3]) == Instance
+    assert type(lf.instances[0]) is PredictedInstance
+    assert type(lf.instances[1]) is PredictedInstance
+    assert type(lf.instances[2]) is Instance
+    assert type(lf.instances[3]) is Instance
     assert lf.instances[2].from_predicted == lf.instances[1]
     assert lf.instances[3].from_predicted == lf.instances[0]
     assert len(lf.unused_predictions) == 0
@@ -114,7 +134,7 @@ def test_read_instances_from_predicted(slp_real_data):
 
 def test_read_labels_multiview(slp_multiview):
     labels = read_labels(slp_multiview)
-    assert type(labels) == Labels
+    assert type(labels) is Labels
     assert len(labels.sessions) == 1
     assert isinstance(labels.sessions[0], RecordingSession)
 
@@ -140,10 +160,12 @@ def test_read_skeleton(centered_pair):
     skeletons = read_skeletons(centered_pair)
     assert len(skeletons) == 1
     skeleton = skeletons[0]
-    assert type(skeleton) == Skeleton
+    assert type(skeleton) is Skeleton
     assert len(skeleton.nodes) == 24
     assert len(skeleton.edges) == 23
-    assert len(skeleton.symmetries) == 20
+    # Legacy files may have duplicate symmetries (one for each direction)
+    # After deduplication, we should have 10 unique symmetry pairs
+    assert len(skeleton.symmetries) == 10
     assert "wingR" in skeleton.symmetry_names[0]
     assert "wingL" in skeleton.symmetry_names[0]
 
@@ -365,7 +387,7 @@ def test_write_labels(centered_pair, slp_real_data, tmp_path):
         assert [len(lf) for lf in saved_labels] == [len(lf) for lf in labels]
         np.testing.assert_array_equal(saved_labels.numpy(), labels.numpy())
         assert saved_labels.video.filename == labels.video.filename
-        assert type(saved_labels.video.backend) == type(labels.video.backend)
+        assert type(saved_labels.video.backend) is type(labels.video.backend)
         assert saved_labels.video.backend.grayscale == labels.video.backend.grayscale
         assert saved_labels.video.backend.shape == labels.video.backend.shape
         assert len(saved_labels.skeletons) == len(labels.skeletons) == 1
@@ -645,14 +667,17 @@ def test_make_session_and_session_to_dict(
     """Test recording session (de)serialization functions.
 
     Args:
-        frame_group_345: Frame group with an `InstanceGroup` at each camera view.
+        recording_session_345: A RecordingSession object with 3 cameras, 4 videos,
+            and 5 frame groups.
     """
 
     def assert_cameras_equal(camera: Camera, camera_0: Camera):
         """Compare two cameras.
+
         Args:
             camera: First camera.
             camera_0: Second camera.
+
         Raises:
             AssertionError: If the cameras are not equal.
         """
@@ -806,12 +831,12 @@ def test_load_multi_skeleton(tmpdir):
 
 def test_slp_imgvideo(tmpdir, slp_imgvideo):
     labels = read_labels(slp_imgvideo)
-    assert type(labels.video.backend) == ImageVideo
+    assert type(labels.video.backend) is ImageVideo
     assert labels.video.shape == (3, 384, 384, 1)
 
     write_labels(tmpdir / "test.slp", labels)
     labels = read_labels(tmpdir / "test.slp")
-    assert type(labels.video.backend) == ImageVideo
+    assert type(labels.video.backend) is ImageVideo
     assert labels.video.shape == (3, 384, 384, 1)
 
     videos = [Video.from_filename(["fake1.jpg", "fake2.jpg"])]
@@ -819,7 +844,7 @@ def test_slp_imgvideo(tmpdir, slp_imgvideo):
     assert len(videos[0].filename) == 2
     write_videos(tmpdir / "test2.slp", videos)
     videos = read_videos(tmpdir / "test2.slp")
-    assert type(videos[0].backend) == ImageVideo
+    assert type(videos[0].backend) is ImageVideo
     assert len(videos[0].filename) == 2
     assert videos[0].shape is None
 
@@ -841,16 +866,50 @@ def test_suggestions(tmpdir):
     assert len(loaded_suggestions) == 0
 
 
+def test_suggestions_metadata(tmpdir):
+    """Test that suggestion metadata (e.g., group) is preserved during read/write."""
+    labels = Labels()
+    labels.videos.append(Video.from_filename("fake.mp4"))
+
+    # Create suggestions with different group values in metadata
+    labels.suggestions.append(
+        SuggestionFrame(video=labels.video, frame_idx=0, metadata={"group": 0})
+    )
+    labels.suggestions.append(
+        SuggestionFrame(video=labels.video, frame_idx=1, metadata={"group": 1})
+    )
+    labels.suggestions.append(
+        SuggestionFrame(video=labels.video, frame_idx=2, metadata={"group": 2})
+    )
+
+    # Write and read suggestions
+    write_suggestions(tmpdir / "test.slp", labels.suggestions, labels.videos)
+    loaded_suggestions = read_suggestions(tmpdir / "test.slp", labels.videos)
+
+    # Verify metadata is preserved
+    assert len(loaded_suggestions) == 3
+    assert loaded_suggestions[0].metadata["group"] == 0
+    assert loaded_suggestions[1].metadata["group"] == 1
+    assert loaded_suggestions[2].metadata["group"] == 2
+
+    # Test backward compatibility: suggestions without metadata default to group 0
+    suggestion_no_metadata = SuggestionFrame(video=labels.video, frame_idx=3)
+    write_suggestions(tmpdir / "test2.slp", [suggestion_no_metadata], labels.videos)
+    loaded_suggestions = read_suggestions(tmpdir / "test2.slp", labels.videos)
+    assert len(loaded_suggestions) == 1
+    assert loaded_suggestions[0].metadata["group"] == 0
+
+
 def test_pkg_roundtrip(tmpdir, slp_minimal_pkg):
     labels = read_labels(slp_minimal_pkg)
-    assert type(labels.video.backend) == HDF5Video
+    assert type(labels.video.backend) is HDF5Video
     assert labels.video.shape == (1, 384, 384, 1)
     assert labels.video.backend.embedded_frame_inds == [0]
     assert labels.video.filename == slp_minimal_pkg
 
     write_labels(str(tmpdir / "roundtrip.pkg.slp"), labels)
     labels = read_labels(str(tmpdir / "roundtrip.pkg.slp"))
-    assert type(labels.video.backend) == HDF5Video
+    assert type(labels.video.backend) is HDF5Video
     assert labels.video.shape == (1, 384, 384, 1)
     assert labels.video.backend.embedded_frame_inds == [0]
     assert (
@@ -864,7 +923,7 @@ def test_pkg_roundtrip(tmpdir, slp_minimal_pkg):
 )
 def test_embed(tmpdir, slp_real_data, to_embed):
     base_labels = read_labels(slp_real_data)
-    assert type(base_labels.video.backend) == MediaVideo
+    assert type(base_labels.video.backend) is MediaVideo
     assert (
         Path(base_labels.video.filename).as_posix()
         == "tests/data/videos/centered_pair_low_quality.mp4"
@@ -878,7 +937,7 @@ def test_embed(tmpdir, slp_real_data, to_embed):
     write_labels(labels_path, base_labels, embed=to_embed)
     labels = read_labels(labels_path)
     assert len(labels) == 10
-    assert type(labels.video.backend) == HDF5Video
+    assert type(labels.video.backend) is HDF5Video
     assert Path(labels.video.filename).as_posix() == labels_path
     assert (
         Path(labels.video.source_video.filename).as_posix()
@@ -916,7 +975,6 @@ def test_embed_hdf5_format(tmpdir, slp_real_data):
     ]
 
     # Prepare metadata and process with HDF5 format
-    from sleap_io.io.slp import prepare_frames_to_embed, process_and_embed_frames
 
     frames_metadata = prepare_frames_to_embed(labels_path, base_labels, frames_to_embed)
     replaced_videos = process_and_embed_frames(
@@ -942,7 +1000,7 @@ def test_embed_hdf5_format(tmpdir, slp_real_data):
 
     # Verify the embedded file
     labels = read_labels(labels_path)
-    assert type(labels.video.backend) == HDF5Video
+    assert type(labels.video.backend) is HDF5Video
     assert Path(labels.video.filename).as_posix() == labels_path
 
     # Check the image format
@@ -961,7 +1019,6 @@ def test_embed_variable_length(tmpdir, slp_real_data):
     ]
 
     # First prepare the metadata
-    from sleap_io.io.slp import prepare_frames_to_embed, process_and_embed_frames
 
     frames_metadata = prepare_frames_to_embed(labels_path, base_labels, frames_to_embed)
 
@@ -989,7 +1046,7 @@ def test_embed_variable_length(tmpdir, slp_real_data):
 
     # Verify the embedded file
     labels = read_labels(labels_path)
-    assert type(labels.video.backend) == HDF5Video
+    assert type(labels.video.backend) is HDF5Video
     assert Path(labels.video.filename).as_posix() == labels_path
 
     # Check that the frames were properly embedded
@@ -1025,7 +1082,7 @@ def test_embed_two_rounds(tmpdir, slp_real_data):
         Path(labels3.video.filename).as_posix()
         == "tests/data/videos/centered_pair_low_quality.mp4"
     )
-    assert type(labels3.video.backend) == MediaVideo
+    assert type(labels3.video.backend) is MediaVideo
 
 
 def test_embed_empty_video(tmpdir, slp_real_data, centered_pair_frame_paths):
@@ -1078,7 +1135,7 @@ def test_embed_grayscale(tmpdir, slp_real_data):
 
 def test_lazy_video_read(slp_real_data):
     labels = read_labels(slp_real_data)
-    assert type(labels.video.backend) == MediaVideo
+    assert type(labels.video.backend) is MediaVideo
     assert labels.video.exists()
 
     labels = read_labels(slp_real_data, open_videos=False)
@@ -1156,15 +1213,17 @@ def conditional_import(verbose):
     with mock.patch("tqdm.tqdm") as mock_tqdm:
         result = mod.conditional_import(True)
         assert result is True  # The import happened
+        # Verify tqdm was imported when verbose=True
+        assert mock_tqdm is not None
 
     # Test with verbose=False
-    with mock.patch("tqdm.tqdm") as mock_tqdm:
+    with mock.patch("tqdm.tqdm"):
         result = mod.conditional_import(False)
         assert result is False  # The import didn't happen
 
 
 def test_embed_frames_verbose_propagation(slp_minimal, tmp_path):
-    """Test that embed_frames propagates the verbose parameter to process_and_embed_frames."""
+    """Test that embed_frames propagates verbose to process_and_embed_frames."""
     labels = load_slp(slp_minimal)
     frames_to_embed = [(labels.videos[0], 0)]
 
@@ -1205,25 +1264,29 @@ def test_embed_videos_verbose_propagation(slp_minimal, tmp_path):
 
 
 def test_write_videos_verbose_propagation(slp_minimal, tmp_path):
-    """Test that write_videos propagates the verbose parameter to process_and_embed_frames."""
+    """Test that write_videos propagates verbose to process_and_embed_frames."""
     labels = load_slp(slp_minimal)
 
     # Create temp file for embedding
     temp_slp = tmp_path / "test_write_videos_prop.slp"
 
-    # Mock process_and_embed_frames to verify verbose is correctly passed when embedding is needed
-    with mock.patch("sleap_io.io.slp.process_and_embed_frames") as mock_embed:
-        # This is a simplified test as we can't easily trigger the condition where write_videos
+    # Mock process_and_embed_frames to verify verbose is correctly passed when
+    # embedding is needed
+    with mock.patch("sleap_io.io.slp.process_and_embed_frames"):
+        # This is a simplified test as we can't easily trigger the condition where
+        # write_videos
         # calls process_and_embed_frames directly
         write_videos(temp_slp, labels.videos, verbose=True)
-        # In a real case with embedded videos, this would verify that verbose is passed correctly
+        # In a real case with embedded videos, this would verify that verbose is
+        # passed correctly
         # Since we're not actually embedding in this test, the mock may not be called
 
-    # The actual test here is that the function accepts the verbose parameter without errors
+    # The actual test here is that the function accepts the verbose parameter
+    # without errors
 
 
 def test_write_labels_verbose_propagation(slp_minimal, tmp_path):
-    """Test that write_labels propagates the verbose parameter to embed_videos and write_videos."""
+    """Test that write_labels propagates verbose to embed_videos and write_videos."""
     labels = load_slp(slp_minimal)
 
     # Create temp file
@@ -1233,7 +1296,6 @@ def test_write_labels_verbose_propagation(slp_minimal, tmp_path):
     with mock.patch("sleap_io.io.slp.embed_videos") as mock_embed_videos, mock.patch(
         "sleap_io.io.slp.write_videos"
     ) as mock_write_videos:
-
         write_labels(temp_slp, labels, embed="user", verbose=True)
 
         # Check that verbose=True was passed to embed_videos
@@ -1246,7 +1308,6 @@ def test_write_labels_verbose_propagation(slp_minimal, tmp_path):
     with mock.patch("sleap_io.io.slp.embed_videos") as mock_embed_videos, mock.patch(
         "sleap_io.io.slp.write_videos"
     ) as mock_write_videos:
-
         write_labels(temp_slp, labels, embed="user", verbose=False)
 
         # Check that verbose=False was passed to embed_videos
@@ -1257,7 +1318,7 @@ def test_write_labels_verbose_propagation(slp_minimal, tmp_path):
 
 
 def test_format_id_1_3_tracking_score(tmp_path):
-    """Test that FORMAT_ID 1.3 properly handles tracking_score field."""
+    """Test that current FORMAT_ID properly handles tracking_score field."""
     # Create test data with tracking scores
     skeleton = Skeleton(["A", "B", "C"])
     track = Track("track1")
@@ -1281,13 +1342,13 @@ def test_format_id_1_3_tracking_score(tmp_path):
         videos=[video], skeletons=[skeleton], tracks=[track], labeled_frames=[lf]
     )
 
-    # Save with FORMAT_ID 1.3
-    test_path = tmp_path / "test_format_1_3.slp"
+    # Save with current FORMAT_ID
+    test_path = tmp_path / "test_format_tracking_score.slp"
     write_labels(test_path, labels)
 
-    # Verify FORMAT_ID is 1.3
+    # Verify FORMAT_ID is current (1.4)
     format_id = read_hdf5_attrs(test_path, "metadata", "format_id")
-    assert format_id == 1.3
+    assert format_id == 1.4
 
     # Load and verify tracking scores are preserved
     loaded_labels = read_labels(test_path)
@@ -1545,7 +1606,10 @@ def test_mixed_video_scenarios(tmp_path, centered_pair_low_quality_video):
 
 
 def test_save_overwrite_without_embedded(tmp_path, centered_pair_low_quality_video):
-    """Test that saving with embed=False over the same file works when no embedded data."""
+    """Test saving with embed=False over the same file.
+
+    Tests when no embedded data exists.
+    """
     # Create test data with external video
     skeleton = Skeleton(["A", "B"])
     video = centered_pair_low_quality_video
@@ -1572,7 +1636,7 @@ def test_save_overwrite_without_embedded(tmp_path, centered_pair_low_quality_vid
 
 
 def test_save_overwrite_embedded_with_source(tmp_path, centered_pair_low_quality_video):
-    """Test that saving with embed=False over the same file works when embedded data has source."""
+    """Test saving with embed=False over same file when embedded data has source."""
     # Create test data
     skeleton = Skeleton(["A", "B"])
     video = centered_pair_low_quality_video
@@ -1610,9 +1674,6 @@ def test_self_referential_path_detection(tmp_path):
     # Create instance and labeled frame
     inst = Instance([[1, 2], [3, 4]], skeleton=skeleton, track=track)
     lf = LabeledFrame(video=video, frame_idx=0, instances=[inst])
-    labels = Labels(
-        videos=[video], skeletons=[skeleton], tracks=[track], labeled_frames=[lf]
-    )
 
     # Create a .pkg.slp file with embedded frames
     pkg_path = tmp_path / "test.pkg.slp"
@@ -1625,7 +1686,7 @@ def test_self_referential_path_detection(tmp_path):
 
     # Create a mock embedded video with minimal HDF5Video backend
     # We need to use a mock to bypass the actual file reading
-    from unittest.mock import Mock, patch
+    from unittest.mock import patch
 
     # Create a real HDF5Video instance but mock its file operations
     with patch("h5py.File"):
@@ -1694,7 +1755,7 @@ def test_labels_save_restore_original_videos_api(tmp_path, slp_minimal_pkg):
     labels_preserve = load_file(output_preserve)
     assert labels_preserve.videos[0].filename == slp_minimal_pkg
     assert labels_preserve.videos[0].backend_metadata["type"] == "HDF5Video"
-    assert labels_preserve.videos[0].backend_metadata["has_embedded_images"] == True
+    assert labels_preserve.videos[0].backend_metadata["has_embedded_images"] is True
 
 
 def test_save_slp_restore_original_videos_api(tmp_path, slp_minimal_pkg):
@@ -1729,7 +1790,6 @@ def test_video_metadata_preservation(tmp_path, slp_minimal_pkg):
     # Store original metadata for comparison
     # For minimal_instance.pkg.slp, source_video IS the original video
     original_backend_metadata = video.source_video.backend_metadata.copy()
-    source_backend_metadata = video.backend_metadata.copy()
 
     # Test EMBED mode
     labels = load_file(slp_minimal_pkg)  # Fresh load
@@ -1898,6 +1958,73 @@ def test_video_to_dict_with_none_backend(tmp_path):
     assert video_dict["backend"] == video.backend_metadata
 
 
+def test_video_to_dict_tiffvideo(tmp_path, multipage_tiff_path):
+    """Test that video_to_dict handles TiffVideo backend correctly."""
+    from sleap_io.io.video_reading import TiffVideo
+
+    # Create a TiffVideo backend
+    backend = TiffVideo(filename=multipage_tiff_path, grayscale=True, keep_open=False)
+    video = Video(
+        filename=multipage_tiff_path,
+        backend=backend,
+    )
+
+    video_dict = video_to_dict(video, labels_path=tmp_path / "test.slp")
+
+    assert video_dict["filename"] == multipage_tiff_path
+    assert video_dict["backend"]["type"] == "TiffVideo"
+    assert video_dict["backend"]["shape"] == video.shape
+    assert video_dict["backend"]["grayscale"] is True
+    assert video_dict["backend"]["keep_open"] is False
+    assert video_dict["backend"]["filename"] == multipage_tiff_path
+    assert video_dict["backend"]["format"] == "multi_page"
+
+
+def test_tiffvideo_roundtrip(tmp_path, multipage_tiff_path):
+    """Test saving and loading Labels with TiffVideo backend."""
+    from sleap_io.io.video_reading import TiffVideo
+
+    # Create labels with a TiffVideo
+    video = Video.from_filename(multipage_tiff_path)
+    assert isinstance(video.backend, TiffVideo)
+
+    skeleton = Skeleton(nodes=["A", "B"])
+    labeled_frame = LabeledFrame(
+        video=video,
+        frame_idx=0,
+        instances=[
+            Instance.from_numpy(
+                points_data=np.array([[10, 20], [30, 40]]),
+                skeleton=skeleton,
+            )
+        ],
+    )
+
+    labels = Labels(
+        videos=[video],
+        skeletons=[skeleton],
+        labeled_frames=[labeled_frame],
+    )
+
+    # Save to SLP
+    output_path = tmp_path / "test_tiff.slp"
+    labels.save(output_path)
+
+    # Load and verify
+    loaded_labels = load_file(output_path)
+    loaded_video = loaded_labels.videos[0]
+
+    # Check that the video was loaded correctly
+    assert loaded_video.filename == multipage_tiff_path
+    assert isinstance(loaded_video.backend, TiffVideo)
+    assert loaded_video.backend.num_frames == 8
+    assert loaded_video.shape == video.shape
+
+    # Verify we can read frames
+    frame = loaded_video.backend.get_frame(0)
+    assert frame.shape == (128, 128, 1)
+
+
 def test_video_original_video_field(slp_minimal_pkg):
     """Test that Video objects have the new original_video field."""
     labels = load_file(slp_minimal_pkg)
@@ -1908,7 +2035,8 @@ def test_video_original_video_field(slp_minimal_pkg):
     assert hasattr(video, "original_video")
     assert video.original_video is None  # Not set for current files
 
-    # TODO: The source_video should become original_video when the field rename is complete
+    # TODO: The source_video should become original_video when the field rename
+    # is complete
     assert video.source_video is not None  # This is current behavior
 
 
@@ -1939,14 +2067,15 @@ def test_complex_workflow(tmp_path, slp_minimal_pkg):
     loaded_predictions = load_file(predictions_output)
     assert loaded_predictions.videos[0].filename == train_pkg.as_posix()
     assert loaded_predictions.videos[0].backend_metadata["type"] == "HDF5Video"
-    assert loaded_predictions.videos[0].backend_metadata["has_embedded_images"] == True
+    assert loaded_predictions.videos[0].backend_metadata["has_embedded_images"] is True
 
     # Verify metadata preservation through the workflow
     # The video objects from inference_labels already have source_video metadata
     # which is preserved when we create the predictions Labels object
     assert loaded_predictions.videos[0].source_video is not None
 
-    # The source_video should point to minimal_instance.pkg.slp (the original training data)
+    # The source_video should point to minimal_instance.pkg.slp
+    # (the original training data)
     # This is correct because we're using the same video objects from inference_labels
     assert loaded_predictions.videos[0].source_video.filename == slp_minimal_pkg
     assert (
@@ -1964,10 +2093,11 @@ def test_complex_workflow(tmp_path, slp_minimal_pkg):
 
 def test_write_videos_backwards_compatibility():
     """Test backwards compatibility with restore_source parameter."""
-    from sleap_io.io.slp import write_videos, VideoReferenceMode
-    from sleap_io.model.video import Video
     import tempfile
 
+    from sleap_io.io.slp import write_videos
+    from sleap_io.model.video import Video
+
     video = Video(
         filename="test.mp4",
         backend_metadata={
@@ -1981,7 +2111,8 @@ def test_write_videos_backwards_compatibility():
     with tempfile.TemporaryDirectory() as tmpdir:
         output = Path(tmpdir) / "test.slp"
 
-        # Test restore_source=True with reference_mode=None (should use RESTORE_ORIGINAL)
+        # Test restore_source=True with reference_mode=None
+        # (should use RESTORE_ORIGINAL)
         write_videos(str(output), [video], restore_source=True, reference_mode=None)
 
         # Test restore_source=False with reference_mode=None (should use EMBED)
@@ -1990,10 +2121,11 @@ def test_write_videos_backwards_compatibility():
 
 def test_video_lineage_edge_cases():
     """Test edge cases in video lineage metadata handling."""
-    from sleap_io.io.slp import write_videos, VideoReferenceMode
-    from sleap_io.model.video import Video
     import tempfile
 
+    from sleap_io.io.slp import VideoReferenceMode, write_videos
+    from sleap_io.model.video import Video
+
     # Test case 1: Video with original_video already set
     original = Video(
         filename="original.mp4",
@@ -2056,3 +2188,330 @@ def test_video_lineage_edge_cases():
             reference_mode=VideoReferenceMode.EMBED,
             original_videos=[video_with_original, video_with_source_original],
         )
+
+
+# Tests for read_labels_set
+
+
+def test_read_labels_set_from_directory(tmp_path, slp_minimal):
+    """Test loading LabelsSet from a directory of SLP files."""
+    # Load minimal labels
+    labels = load_slp(slp_minimal)
+
+    # Create test directory with multiple SLP files
+    test_dir = tmp_path / "splits"
+    test_dir.mkdir()
+
+    # Save splits
+    labels.save(test_dir / "train.slp", embed=False)
+    labels.save(test_dir / "val.slp", embed=False)
+    labels.save(test_dir / "test.slp", embed=False)
+
+    # Load as LabelsSet
+    labels_set = read_labels_set(test_dir)
+
+    assert isinstance(labels_set, LabelsSet)
+    assert len(labels_set) == 3
+    assert "train" in labels_set
+    assert "val" in labels_set
+    assert "test" in labels_set
+
+    # Check that each loaded Labels has correct data
+    for name in ["train", "val", "test"]:
+        assert len(labels_set[name]) == len(labels)
+
+
+def test_read_labels_set_from_list(tmp_path, slp_minimal):
+    """Test loading LabelsSet from a list of file paths."""
+    labels = load_slp(slp_minimal)
+
+    # Create test files
+    file1 = tmp_path / "split1.slp"
+    file2 = tmp_path / "split2.slp"
+    labels.save(file1, embed=False)
+    labels.save(file2, embed=False)
+
+    # Load from list
+    labels_set = read_labels_set([file1, file2])
+
+    assert isinstance(labels_set, LabelsSet)
+    assert len(labels_set) == 2
+    assert "split1" in labels_set
+    assert "split2" in labels_set
+
+
+def test_read_labels_set_from_dict(tmp_path, slp_minimal):
+    """Test loading LabelsSet from a dictionary mapping."""
+    labels = load_slp(slp_minimal)
+
+    # Create test files
+    train_file = tmp_path / "train_data.slp"
+    val_file = tmp_path / "validation_data.slp"
+    labels.save(train_file, embed=False)
+    labels.save(val_file, embed=False)
+
+    # Load from dictionary
+    labels_set = read_labels_set({"training": train_file, "validation": val_file})
+
+    assert isinstance(labels_set, LabelsSet)
+    assert len(labels_set) == 2
+    assert "training" in labels_set
+    assert "validation" in labels_set
+
+
+def test_read_labels_set_empty_directory(tmp_path):
+    """Test error handling for empty directory."""
+    empty_dir = tmp_path / "empty"
+    empty_dir.mkdir()
+
+    with pytest.raises(ValueError, match="No .slp files found"):
+        read_labels_set(empty_dir)
+
+
+def test_read_labels_set_invalid_path(tmp_path):
+    """Test error handling for invalid path."""
+    # Non-directory path
+    file_path = tmp_path / "file.txt"
+    file_path.write_text("test")
+
+    with pytest.raises(ValueError, match="Path must be a directory"):
+        read_labels_set(file_path)
+
+
+def test_read_labels_set_without_videos(tmp_path, slp_minimal):
+    """Test loading LabelsSet without opening videos."""
+    labels = load_slp(slp_minimal)
+
+    # Save a file
+    test_file = tmp_path / "test.slp"
+    labels.save(test_file, embed=False)
+
+    # Load without opening videos
+    labels_set = read_labels_set([test_file], open_videos=False)
+
+    assert isinstance(labels_set, LabelsSet)
+    assert len(labels_set) == 1
+
+    # Videos should not be opened
+    for lf in labels_set["test"].labeled_frames:
+        assert lf.video is not None
+        assert not lf.video.is_open
+
+
+def test_read_labels_set_string_paths(tmp_path, slp_minimal):
+    """Test that string paths work as well as Path objects."""
+    labels = load_slp(slp_minimal)
+
+    # Create test directory
+    test_dir = tmp_path / "splits"
+    test_dir.mkdir()
+    labels.save(test_dir / "data.slp", embed=False)
+
+    # Test with string path
+    labels_set = read_labels_set(str(test_dir))
+    assert len(labels_set) == 1
+    assert "data" in labels_set
+
+
+@pytest.mark.parametrize("encode_plugin", ["opencv", "imageio"])
+@pytest.mark.parametrize("decode_plugin", ["opencv", "imageio"])
+def test_embed_channel_order_consistency(
+    tmp_path, small_robot_video, encode_plugin, decode_plugin
+):
+    """Test frames match exactly regardless of encoding/decoding plugin.
+
+    This test verifies that the RGB/BGR channel order handling works correctly by:
+    1. Encoding frames with one plugin (opencv=BGR or imageio=RGB)
+    2. Decoding frames with a different plugin
+    3. Verifying frames match the original exactly (automatic conversion)
+
+    Args:
+        tmp_path: Pytest temporary directory fixture.
+        small_robot_video: Test video fixture (3-frame robot video).
+        encode_plugin: Plugin to use for encoding ("opencv" or "imageio").
+        decode_plugin: Plugin to use for decoding ("opencv" or "imageio").
+    """
+    # Step 1: Extract original frames from the video
+    original_frames = []
+    for i in range(len(small_robot_video)):
+        frame = small_robot_video[i]
+        original_frames.append(frame.copy())
+
+    # Step 2: Create Labels with labeled instances for all frames
+    skeleton = Skeleton(nodes=["node1", "node2"])
+    labeled_frames = []
+    for i in range(len(small_robot_video)):
+        instance = Instance(
+            points=np.array([[10.0 + i, 20.0 + i], [30.0 + i, 40.0 + i]]),
+            skeleton=skeleton,
+        )
+        labeled_frame = LabeledFrame(
+            video=small_robot_video, frame_idx=i, instances=[instance]
+        )
+        labeled_frames.append(labeled_frame)
+
+    labels = Labels(
+        videos=[small_robot_video], skeletons=[skeleton], labeled_frames=labeled_frames
+    )
+
+    # Step 3: Save to .pkg.slp with specified encoding plugin
+    pkg_path = tmp_path / f"test_{encode_plugin}.pkg.slp"
+
+    # Store original default and set encoding plugin
+    original_default = get_default_image_plugin()
+    try:
+        set_default_image_plugin(encode_plugin)
+        save_slp(labels, str(pkg_path), embed="all", plugin=encode_plugin)
+    finally:
+        # Restore original default
+        set_default_image_plugin(original_default)
+
+    # Step 4: Verify the channel_order attribute was stored correctly in HDF5
+    with h5py.File(pkg_path, "r") as f:
+        video_ds = f["video0/video"]
+        assert "channel_order" in video_ds.attrs
+        expected_channel_order = "BGR" if encode_plugin == "opencv" else "RGB"
+        assert video_ds.attrs["channel_order"] == expected_channel_order
+
+    # Step 5: Load with specified decoding plugin
+    # We control the decoder by mocking sys.modules and setting the plugin preference
+    try:
+        set_default_image_plugin(decode_plugin)
+
+        # Mock sys.modules to ensure the decode plugin is used
+        if decode_plugin == "opencv":
+            # Ensure cv2 is in sys.modules (it should be already)
+            import cv2  # noqa: F401
+
+        loaded_labels = load_slp(str(pkg_path))
+        loaded_video = loaded_labels.videos[0]
+
+        # Step 6: Extract embedded frames
+        embedded_frames = []
+        for i in range(len(loaded_video)):
+            frame = loaded_video[i]
+            embedded_frames.append(frame.copy())
+
+    finally:
+        # Restore original default
+        set_default_image_plugin(original_default)
+
+    # Step 7: Compare frames - should match exactly regardless of plugin
+    assert len(original_frames) == len(embedded_frames)
+    for i in range(len(original_frames)):
+        orig = original_frames[i]
+        embd = embedded_frames[i]
+
+        # Frames should match exactly (pixel-perfect)
+        assert np.array_equal(orig, embd), (
+            f"Frame {i} mismatch (encode={encode_plugin}, decode={decode_plugin})"
+        )
+
+        # Additional check: ensure no accidental channel swap
+        if orig.shape[-1] == 3:
+            # If channels were swapped, they should NOT match
+            orig_reversed = orig[..., ::-1]
+            if not np.array_equal(
+                orig, orig_reversed
+            ):  # Only if image is not symmetric
+                assert not np.array_equal(orig_reversed, embd), (
+                    f"Frame {i} has reversed channels!"
+                )
+
+
+@pytest.mark.parametrize("plugin", ["opencv", "imageio"])
+def test_embed_channel_order_metadata(tmp_path, small_robot_video, plugin):
+    """Test that channel_order metadata is correctly stored in HDF5 datasets.
+
+    Args:
+        tmp_path: Pytest temporary directory fixture.
+        small_robot_video: Test video fixture (3-frame robot video).
+        plugin: Plugin to use for encoding ("opencv" or "imageio").
+    """
+    # Create minimal Labels
+    skeleton = Skeleton(nodes=["node1"])
+    instance = Instance(points=np.array([[10.0, 20.0]]), skeleton=skeleton)
+    labeled_frame = LabeledFrame(
+        video=small_robot_video, frame_idx=0, instances=[instance]
+    )
+    labels = Labels(
+        videos=[small_robot_video], skeletons=[skeleton], labeled_frames=[labeled_frame]
+    )
+
+    # Save with specified plugin
+    pkg_path = tmp_path / f"test_{plugin}.pkg.slp"
+    original_default = get_default_image_plugin()
+    try:
+        set_default_image_plugin(plugin)
+        save_slp(labels, str(pkg_path), embed="all", plugin=plugin)
+    finally:
+        set_default_image_plugin(original_default)
+
+    # Verify HDF5 metadata
+    with h5py.File(pkg_path, "r") as f:
+        video_ds = f["video0/video"]
+
+        # Check that channel_order attribute exists
+        assert "channel_order" in video_ds.attrs, "channel_order attribute missing"
+
+        # Check that it has the correct value
+        expected_channel_order = "BGR" if plugin == "opencv" else "RGB"
+        actual_channel_order = video_ds.attrs["channel_order"]
+        assert actual_channel_order == expected_channel_order, (
+            f"Expected {expected_channel_order}, got {actual_channel_order}"
+        )
+
+        # Also verify format_id is 1.4+
+        assert "metadata" in f
+        assert "format_id" in f["metadata"].attrs
+        format_id = f["metadata"].attrs["format_id"]
+        assert format_id >= 1.4, f"Expected format_id >= 1.4, got {format_id}"
+
+
+def test_embed_backwards_compatibility_channel_order(tmp_path, small_robot_video):
+    """Test that legacy files (format < 1.4) without channel_order default to BGR.
+
+    This ensures backwards compatibility with files created before the channel_order
+    attribute was added.
+
+    Args:
+        tmp_path: Pytest temporary directory fixture.
+        small_robot_video: Test video fixture (3-frame robot video).
+    """
+    # Create and save a file with current format
+    skeleton = Skeleton(nodes=["node1"])
+    instance = Instance(points=np.array([[10.0, 20.0]]), skeleton=skeleton)
+    labeled_frame = LabeledFrame(
+        video=small_robot_video, frame_idx=0, instances=[instance]
+    )
+    labels = Labels(
+        videos=[small_robot_video], skeletons=[skeleton], labeled_frames=[labeled_frame]
+    )
+
+    pkg_path = tmp_path / "legacy.pkg.slp"
+    save_slp(labels, str(pkg_path), embed="all")
+
+    # Manually modify the HDF5 file to simulate a legacy file
+    with h5py.File(pkg_path, "r+") as f:
+        # Remove channel_order attribute
+        video_ds = f["video0/video"]
+        if "channel_order" in video_ds.attrs:
+            del video_ds.attrs["channel_order"]
+
+        # Set format_id to 1.3 (pre-channel_order)
+        f["metadata"].attrs["format_id"] = 1.3
+
+    # Load the file and verify it defaults to BGR
+    loaded_labels = load_slp(str(pkg_path))
+    loaded_video = loaded_labels.videos[0]
+
+    # The HDF5Video backend should have defaulted channel_order to BGR
+    assert hasattr(loaded_video.backend, "channel_order")
+    assert loaded_video.backend.channel_order == "BGR", (
+        "Legacy files should default to BGR"
+    )
+
+    # Verify we can still read frames without errors
+    frame = loaded_video[0]
+    assert frame is not None
+    assert frame.shape[-1] == 3  # RGB image
