@@ -1,26 +1,36 @@
 """Tests for sleap_io.io.ultralytics module."""
 
+import tempfile
+from pathlib import Path
+
+import imageio.v3 as iio
 import numpy as np
 import pytest
-from pathlib import Path
-import tempfile
 import yaml
-import imageio.v3 as iio
-import sleap_io
 
-from sleap_io import Labels, Skeleton, Node, Edge, Instance, LabeledFrame, Track, Video
+import sleap_io
+from sleap_io import (
+    Edge,
+    Instance,
+    LabeledFrame,
+    Labels,
+    LabelsSet,
+    Node,
+    Skeleton,
+    Video,
+)
+from sleap_io.io.main import load_file, load_ultralytics, save_file, save_ultralytics
 from sleap_io.io.ultralytics import (
-    read_labels,
-    write_labels,
+    create_skeleton_from_config,
+    denormalize_coordinates,
+    normalize_coordinates,
     parse_data_yaml,
-    create_skeleton_from_config,
     parse_label_file,
+    read_labels,
+    read_labels_set,
     write_label_file,
-    create_data_yaml,
-    normalize_coordinates,
-    denormalize_coordinates,
+    write_labels,
 )
-from sleap_io.io.main import load_file, save_file, load_ultralytics, save_ultralytics
 
 
 def test_parse_data_yaml(ultralytics_data_yaml):
@@ -296,11 +306,11 @@ def test_denormalize_coordinates():
 
     assert points_array[0, 0] == 100.0  # 0.125 * 800
     assert points_array[0, 1] == 200.0  # 0.5 * 400
-    assert points_array[0, 2] == True
+    assert points_array[0, 2] == 1.0  # True converted to float
 
     assert np.isnan(points_array[1, 0])
     assert np.isnan(points_array[1, 1])
-    assert points_array[1, 2] == False
+    assert points_array[1, 2] == 0.0  # False converted to float
 
 
 def test_load_file_integration(ultralytics_dataset):
@@ -436,7 +446,10 @@ def test_keypoint_count_mismatch_warning(tmp_path):
 
 
 def test_realistic_labels_roundtrip(tmp_path, labels_predictions):
-    """Test roundtrip conversion with realistic labels containing multiple instances and tracks."""
+    """Test roundtrip conversion with realistic labels.
+
+    Tests labels containing multiple instances and tracks.
+    """
     # Take a subset of frames for testing
     labels_subset = Labels(
         labeled_frames=labels_predictions.labeled_frames[:10],
@@ -453,7 +466,8 @@ def test_realistic_labels_roundtrip(tmp_path, labels_predictions):
     image_files = list((output_dir / "train" / "images").glob("*.png"))
     assert len(image_files) == 10
 
-    # Read back and compare (note: tracks will be different as Ultralytics doesn't preserve them)
+    # Read back and compare
+    # Note: tracks will be different as Ultralytics doesn't preserve them
     labels_read = read_labels(output_dir, split="train")
     assert len(labels_read.labeled_frames) == 10
     assert len(labels_read.skeletons) == 1
@@ -559,7 +573,10 @@ def test_multiprocessing_write(tmp_path, centered_pair_low_quality_video):
     # Write with multiprocessing
     output_dir = tmp_path / "ultralytics_multiproc"
     write_labels(
-        labels, str(output_dir), use_multiprocessing=True, n_workers=2, verbose=False
+        labels,
+        str(output_dir),
+        use_multiprocessing=True,
+        n_workers=2,
     )
 
     # Verify all images were created
@@ -694,7 +711,7 @@ def test_missing_labels_directory(tmp_path):
 def test_video_shape_none_fallback(tmp_path, centered_pair_low_quality_video):
     """Test fallback to reading frame when video.shape is None."""
     # Create a mock video with None shape
-    from unittest.mock import Mock, patch
+    from unittest.mock import Mock
 
     # Create test setup
     skeleton = Skeleton([Node("a"), Node("b")])
@@ -730,7 +747,7 @@ def test_empty_skeletons_error(tmp_path):
 
 def test_frame_image_none_warning(tmp_path, centered_pair_low_quality_video):
     """Test warning when frame.image returns None."""
-    from unittest.mock import Mock, PropertyMock, patch
+    from unittest.mock import PropertyMock, patch
 
     skeleton = Skeleton([Node("a"), Node("b")])
     instance = Instance.from_numpy(
@@ -845,6 +862,7 @@ def test_parse_label_file_edge_cases(tmp_path, ultralytics_skeleton):
 def test_frame_none_return_from_video(tmp_path):
     """Test handling when video returns None for a frame."""
     from unittest.mock import Mock, patch
+
     from sleap_io.io.ultralytics import _save_frame_image
 
     # Create a mock video that returns None for frame
@@ -880,7 +898,10 @@ def test_parse_label_file_with_invalid_keypoint_count(tmp_path):
 
 
 def test_read_labels_fallback_to_read_first_frame(tmp_path):
-    """Test read_labels when video.shape is None and falls back to reading first frame."""
+    """Test read_labels when video.shape is None.
+
+    Tests fallback to reading first frame.
+    """
     from unittest.mock import Mock, patch
 
     # Create test data structure
@@ -932,7 +953,8 @@ def test_read_labels_fallback_to_read_first_frame(tmp_path):
 def test_write_labels_frame_extraction_error_continue(tmp_path):
     """Test write_labels warning for frame.image returning None - code coverage only."""
     # This test just ensures the warning code path is covered
-    # The actual behavior is tricky to test due to how enumerate works with skipped frames
+    # The actual behavior is tricky to test due to how enumerate works with
+    # skipped frames
 
     # We've already tested this code path indirectly in other tests
     # This specific line (291-294) gets executed when frame.image returns None
@@ -1004,3 +1026,223 @@ def test_write_label_file_skip_wrong_points_count(tmp_path):
     # File should be empty
     with open(label_file) as f:
         assert f.read() == ""
+
+
+# Tests for read_labels_set
+
+
+def create_test_ultralytics_dataset(base_path: Path, splits=["train", "val", "test"]):
+    """Create a minimal Ultralytics dataset structure for testing."""
+    # Create data.yaml
+    data_config = {
+        "path": str(base_path),
+        "train": "train/images",
+        "val": "val/images",
+        "test": "test/images",
+        "kpt_shape": [3, 2],  # 3 keypoints
+        "names": ["animal"],
+    }
+
+    with open(base_path / "data.yaml", "w") as f:
+        yaml.dump(data_config, f)
+
+    # Create splits
+    for split in splits:
+        split_path = base_path / split
+        images_path = split_path / "images"
+        labels_path = split_path / "labels"
+
+        images_path.mkdir(parents=True)
+        labels_path.mkdir(parents=True)
+
+        # Create a few fake images and labels
+        for i in range(2):
+            # Create dummy image file (needs to be real image)
+            img_file = images_path / f"img_{i:03d}.jpg"
+            dummy_img = np.zeros((10, 10, 3), dtype=np.uint8)
+            iio.imwrite(img_file, dummy_img)
+
+            # Create label file with normalized coordinates
+            label_file = labels_path / f"img_{i:03d}.txt"
+            # Format: class_id x_center y_center width height x1 y1 v1 x2 y2 v2 x3 y3 v3
+            label_line = "0 0.5 0.5 0.4 0.4 0.4 0.4 2 0.5 0.5 2 0.6 0.6 2\n"
+            label_file.write_text(label_line)
+
+
+def test_read_labels_set_basic(tmp_path):
+    """Test basic loading of LabelsSet from Ultralytics dataset."""
+    dataset_path = tmp_path / "yolo_dataset"
+    dataset_path.mkdir()
+
+    create_test_ultralytics_dataset(dataset_path, ["train", "val"])
+
+    # Load LabelsSet
+    labels_set = read_labels_set(str(dataset_path))
+
+    assert isinstance(labels_set, LabelsSet)
+    assert len(labels_set) == 2
+    assert "train" in labels_set
+    assert "val" in labels_set
+
+    # Check that each split has correct structure
+    for split in ["train", "val"]:
+        labels = labels_set[split]
+        assert isinstance(labels, Labels)
+        assert len(labels) == 2  # 2 frames per split
+        assert len(labels.skeletons) == 1
+        assert len(labels.skeletons[0].nodes) == 3  # 3 keypoints
+
+
+def test_read_labels_set_specific_splits(tmp_path):
+    """Test loading specific splits only."""
+    dataset_path = tmp_path / "yolo_dataset"
+    dataset_path.mkdir()
+
+    create_test_ultralytics_dataset(dataset_path, ["train", "val", "test"])
+
+    # Load only train and test
+    labels_set = read_labels_set(
+        str(dataset_path),
+        splits=["train", "test"],
+    )
+
+    assert len(labels_set) == 2
+    assert "train" in labels_set
+    assert "test" in labels_set
+    assert "val" not in labels_set
+
+
+def test_read_labels_set_custom_skeleton(tmp_path):
+    """Test loading with a custom skeleton."""
+    dataset_path = tmp_path / "yolo_dataset"
+    dataset_path.mkdir()
+
+    create_test_ultralytics_dataset(dataset_path, ["train"])
+
+    # Create custom skeleton
+    skeleton = Skeleton([Node(name="head"), Node(name="body"), Node(name="tail")])
+
+    # Load with custom skeleton
+    labels_set = read_labels_set(str(dataset_path), skeleton=skeleton)
+
+    assert len(labels_set["train"].skeletons[0].nodes) == 3
+    assert labels_set["train"].skeletons[0].nodes[0].name == "head"
+    assert labels_set["train"].skeletons[0].nodes[1].name == "body"
+    assert labels_set["train"].skeletons[0].nodes[2].name == "tail"
+
+
+def test_read_labels_set_missing_splits(tmp_path):
+    """Test handling of missing splits."""
+    dataset_path = tmp_path / "yolo_dataset"
+    dataset_path.mkdir()
+
+    create_test_ultralytics_dataset(dataset_path, ["train"])
+
+    # Try to load non-existent splits
+    labels_set = read_labels_set(
+        str(dataset_path),
+        splits=["train", "val", "test"],  # val and test don't exist
+    )
+
+    # Should only load train
+    assert len(labels_set) == 1
+    assert "train" in labels_set
+    assert "val" not in labels_set
+    assert "test" not in labels_set
+
+
+def test_read_labels_set_no_splits(tmp_path):
+    """Test error when no splits are found."""
+    dataset_path = tmp_path / "empty_dataset"
+    dataset_path.mkdir()
+
+    with pytest.raises(ValueError, match="No splits found"):
+        read_labels_set(str(dataset_path))
+
+
+def test_read_labels_set_auto_detect_splits(tmp_path):
+    """Test automatic detection of available splits."""
+    dataset_path = tmp_path / "yolo_dataset"
+    dataset_path.mkdir()
+
+    # Create only train and valid (not val)
+    create_test_ultralytics_dataset(dataset_path, ["train", "valid"])
+
+    # Don't specify splits - should auto-detect
+    labels_set = read_labels_set(str(dataset_path))
+
+    assert len(labels_set) == 2
+    assert "train" in labels_set
+    assert "valid" in labels_set
+
+
+def test_read_labels_set_no_data_yaml(tmp_path):
+    """Test loading without data.yaml requires a skeleton."""
+    dataset_path = tmp_path / "yolo_dataset"
+    dataset_path.mkdir()
+
+    # Create dataset structure without data.yaml
+    train_path = dataset_path / "train"
+    images_path = train_path / "images"
+    labels_path = train_path / "labels"
+    images_path.mkdir(parents=True)
+    labels_path.mkdir(parents=True)
+
+    # Create a simple file (needs to be real image)
+    img_file = images_path / "img_000.jpg"
+    dummy_img = np.zeros((10, 10, 3), dtype=np.uint8)
+    iio.imwrite(img_file, dummy_img)
+    label_file = labels_path / "img_000.txt"
+    label_file.write_text("0 0.5 0.5 0.4 0.4 0.5 0.5 2\n")
+
+    # Since read_labels requires data.yaml, we need to create a minimal one
+    # even when providing a skeleton
+    data_config = {
+        "path": str(dataset_path),
+        "train": "train/images",
+        "kpt_shape": [1, 2],  # 1 keypoint
+        "names": ["animal"],
+    }
+
+    with open(dataset_path / "data.yaml", "w") as f:
+        yaml.dump(data_config, f)
+
+    # Should work with provided skeleton
+    skeleton = Skeleton([Node(name="point")])
+    labels_set = read_labels_set(str(dataset_path), skeleton=skeleton)
+
+    assert len(labels_set) == 1
+    assert "train" in labels_set
+    # The custom skeleton should be used
+    assert labels_set["train"].skeletons[0].nodes[0].name == "point"
+
+
+def test_read_labels_set_roundtrip(tmp_path, slp_minimal):
+    """Test that we can write labels and read them back as LabelsSet."""
+    from sleap_io import load_slp
+
+    # Load test data
+    labels = load_slp(slp_minimal)
+
+    dataset_path = tmp_path / "yolo_dataset"
+
+    # Write labels in ultralytics format
+    write_labels(
+        labels,
+        str(dataset_path),
+        split_ratios={"train": 0.6, "val": 0.2, "test": 0.2},
+        verbose=False,
+    )
+
+    # Read back as LabelsSet
+    labels_set = read_labels_set(str(dataset_path))
+
+    assert isinstance(labels_set, LabelsSet)
+    assert len(labels_set) == 3
+    assert "train" in labels_set
+    assert "val" in labels_set
+    assert "test" in labels_set
+
+    # Total frames should match original (allowing for rounding in splits)
+    total_frames = sum(len(split_labels) for split_labels in labels_set.values())
+    assert abs(total_frames - len(labels)) <= 1
